#ifndef __DELAY_H
#define __DELAY_H 			   
#ifdef __cplusplus
extern "C" {
#endif
#include "global_param.h"
	
/*****************************************************************/
#define STK_CSR			(*((volatile unsigned long *)0xE000E010))
#define STK_RVR			(*((volatile unsigned long *)0xE000E014))
#define STK_CVR			(*((volatile unsigned long *)0xE000E018))
#define STK_CALIB		(*((volatile unsigned long *)0xE000E01C))

#define RELOAD_50HZ 	(uint32_t)480000
#define SYSTICK_MUL		(uint32_t)1
#define SYSTICK_RELOAD	RELOAD_50HZ / SYSTICK_MUL	/*SysTick->LOAD的值为48M/100;这样算的话(1/48M)*SysTick->LOAD=10ms;故最小调度时间就为10ms*/

#define delay_scale 0.1
/*
因为定时器间隔1ms
*/
typedef enum
{
	TIME_100MMS = 1,
	TIME_1MS = 1,	
	TIME_2MS = 2*TIME_1MS,	
	TIME_6MS = 6*TIME_1MS,	
	TIME_10MS = 10*TIME_1MS,	
	TIME_20MS = 20*TIME_1MS,
	TIME_50MS = 50*TIME_1MS,
	TIME_100MS = 100*TIME_1MS,
	TIME_120MS = 120*TIME_1MS,
	TIME_200MS = 200*TIME_1MS,
	TIME_500MS = 500*TIME_1MS,
	TIME_1000MS = 1000*TIME_1MS,
	TIME_2000MS = 2000*TIME_1MS,
	TIME_5000MS = 5000*TIME_1MS,
	TIME_10000MS = 10000*TIME_1MS,
} TimeType_t;
typedef struct
{
	uint32_t time;
}G_Type_t;

#define TON   (1)
#define TOFF  (0)
typedef struct __attribute__((__packed__)) 
{
  uint32_t start   :1;
  uint32_t timout  :1;
  uint32_t cunt    :15;
  uint32_t des     :15;
}TimBitTypeDef;

typedef struct __attribute__((__packed__)) 
{
	uint8_t  start:1;
	uint8_t  status:7;
	uint16_t src;
	uint16_t count;
	uint8_t  flag;
}DlyTypeDef;

#define Timer_abs(a,b) ((a > b)? (a - b):(b - a))
	
#define Systick_Isr  System_BaseTime_Update

void Systick_Init(void);
void System_BaseTime_Update(void);
uint32_t System_Get_Time(void);
int32_t Timer_Is_TimeReach(uint32_t *pre_time,uint32_t width);
int32_t Timer_Is_TimeReach_No_Repeat(uint32_t *pre_time,uint32_t width);
uint32_t Timer_Get_DiffNow(uint32_t start);

void Timer_Delay_Ms(uint16_t ms);
#ifdef __cplusplus
}
#endif
#endif
