/******************************************************************************
*******************************Driver for LD3320*******************************
* | file      	:	LD3320_Driver.c
* |	version			:	V1.0
* | date				:	2020-06-18
* | function		:	Driver for LD3320
******************************************************************************/

#include "LD3320_Driver.h"
#include "w25qxx.h"	 

UDOUBLE nMp3Size = 0;								// mp3 size
UDOUBLE nMp3Pos = 0;								// mp3 current pos
UBYTE   *nMp3name;									// mp3 playing status
UBYTE 	nAsrStatus = 0;							// Record the status of the ASR	
UBYTE 	nLD_Mode = LD_MODE_IDLE;		// Record the status of LD3320(ASR or Play Sound)
UBYTE 	bMp3Play = 0;								// Record the status of audio player
UBYTE 	ucRegVal;										// Interrupt request number
UBYTE 	ucHighInt;									// Intterrupt enable status
UBYTE 	ucLowInt;										// Intterrupt enable status
UBYTE 	ucStatus;										// FIFO status
UBYTE 	ucSPVol = 15; 							// Speaker volume, MAX=31 MIN=0	

static UBYTE	LD_AsrAddKey(void);
static void		LD_Init_Common(void);
static void		LD_ReloadMp3Data(void);
static void LD_ReloadMp3Data_from_flash(void);
static UBYTE	LD_Check_ASRBusyFlag(void);
static void		LD_WriteReg(UBYTE data1,UBYTE data2);
static UBYTE	LD_ReadReg(UBYTE reg_add);
static void		LD_Init_ASR(void);
static UBYTE	LD_AsrRun(void);


#define ITEM_COUNT 31

#define DEF_MUSI1 241246
#define DEF_MUSI2 271183
#define DEF_ROBOT_READY 9261
#define DEF_BAD   10797
#define DEF_RTC_HEAD   16941
#define DEF_RESPOSE   9261
#define DEF_POINT  10413
#define DEF_COMPLETE 7725
#define DEF_MINUTE 8301
#define DEF_AIR_QUALITY 14061
#define DEF_GOOD 10605
#define DEF_FORWARD 6957
#define DEF_BACKWARD 6381
#define DEF_TURNLEFT 6381
#define DEF_TURNRIGHT 6381
#define DEF_AIR_IAQ 16941
#define DEF_CELSIUS 8877
#define DEF_PERCENTAGE 8685
#define DEF_HUMIDITY 7533
#define DEF_TEMPERATURE 7533

#define DEF_ONE 6765
#define DEF_TWO 7149
#define DEF_THREE 11181
#define DEF_FOUR 9069
#define DEF_FIVE 8301
#define DEF_SIX 7917
#define DEF_SEVEN 8301
#define DEF_EIGHT 7149
#define DEF_NINE 8685
#define DEF_TEN 9453
#define DEF_TWENTY 10797

const uint32_t sum_addr = DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE+DEF_POINT+DEF_COMPLETE+DEF_MINUTE+DEF_AIR_QUALITY+DEF_GOOD+DEF_FORWARD+DEF_BACKWARD+DEF_TURNLEFT+DEF_TURNRIGHT+DEF_AIR_IAQ+DEF_CELSIUS+DEF_PERCENTAGE+DEF_HUMIDITY+DEF_TEMPERATURE;
const ST_ASR_DATA pAsrItem[ITEM_COUNT] =
{
    {0 ,0, DEF_MUSI1}, //music1
    {1 ,DEF_MUSI1,DEF_MUSI2}, //music2
    {2 ,DEF_MUSI1+DEF_MUSI2,DEF_ROBOT_READY}, //Robot_ready
    {3 ,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY,DEF_BAD}, //bad
    {4 ,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD,DEF_RTC_HEAD}, //Array_RTC_HEAD
    {5 ,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD,DEF_RESPOSE}, //Array_Respose 
    {6 ,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE,DEF_POINT}, //point
    {7 ,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE+DEF_POINT,DEF_COMPLETE}, //complete
    {8 ,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE+DEF_POINT+DEF_COMPLETE,DEF_MINUTE}, //minute
    {9 ,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE+DEF_POINT+DEF_COMPLETE+DEF_MINUTE,DEF_AIR_QUALITY}, //air_quality
    {10,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE+DEF_POINT+DEF_COMPLETE+DEF_MINUTE+DEF_AIR_QUALITY,DEF_GOOD}, //good
    {11,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE+DEF_POINT+DEF_COMPLETE+DEF_MINUTE+DEF_AIR_QUALITY+DEF_GOOD,DEF_FORWARD}, //forward
    {12,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE+DEF_POINT+DEF_COMPLETE+DEF_MINUTE+DEF_AIR_QUALITY+DEF_GOOD+DEF_FORWARD,DEF_BACKWARD}, //backward
    {13,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE+DEF_POINT+DEF_COMPLETE+DEF_MINUTE+DEF_AIR_QUALITY+DEF_GOOD+DEF_FORWARD+DEF_BACKWARD,DEF_TURNLEFT}, //turnleft
    {14,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE+DEF_POINT+DEF_COMPLETE+DEF_MINUTE+DEF_AIR_QUALITY+DEF_GOOD+DEF_FORWARD+DEF_BACKWARD+DEF_TURNLEFT,DEF_TURNRIGHT}, //turnright
    {15,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE+DEF_POINT+DEF_COMPLETE+DEF_MINUTE+DEF_AIR_QUALITY+DEF_GOOD+DEF_FORWARD+DEF_BACKWARD+DEF_TURNLEFT+DEF_TURNRIGHT,DEF_AIR_IAQ}, //air_iaq
    {16,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE+DEF_POINT+DEF_COMPLETE+DEF_MINUTE+DEF_AIR_QUALITY+DEF_GOOD+DEF_FORWARD+DEF_BACKWARD+DEF_TURNLEFT+DEF_TURNRIGHT+DEF_AIR_IAQ,DEF_CELSIUS}, //CELSIUS
    {17,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE+DEF_POINT+DEF_COMPLETE+DEF_MINUTE+DEF_AIR_QUALITY+DEF_GOOD+DEF_FORWARD+DEF_BACKWARD+DEF_TURNLEFT+DEF_TURNRIGHT+DEF_AIR_IAQ+DEF_CELSIUS,DEF_PERCENTAGE}, //percentage
    {18,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE+DEF_POINT+DEF_COMPLETE+DEF_MINUTE+DEF_AIR_QUALITY+DEF_GOOD+DEF_FORWARD+DEF_BACKWARD+DEF_TURNLEFT+DEF_TURNRIGHT+DEF_AIR_IAQ+DEF_CELSIUS+DEF_PERCENTAGE,DEF_HUMIDITY}, //humidity
    {19,DEF_MUSI1+DEF_MUSI2+DEF_ROBOT_READY+DEF_BAD+DEF_RTC_HEAD+DEF_RESPOSE+DEF_POINT+DEF_COMPLETE+DEF_MINUTE+DEF_AIR_QUALITY+DEF_GOOD+DEF_FORWARD+DEF_BACKWARD+DEF_TURNLEFT+DEF_TURNRIGHT+DEF_AIR_IAQ+DEF_CELSIUS+DEF_PERCENTAGE+DEF_HUMIDITY,DEF_TEMPERATURE}, //temperature

    {20,sum_addr, DEF_ONE}, //one
    {21,sum_addr+ DEF_ONE,DEF_TWO}, //two
    {22,sum_addr+ DEF_ONE+DEF_TWO,DEF_THREE}, //three
    {23,sum_addr+ DEF_ONE+DEF_TWO+DEF_THREE,DEF_FOUR}, //four
    {24,sum_addr+ DEF_ONE+DEF_TWO+DEF_THREE+DEF_FOUR,DEF_FIVE}, //five
    {25,sum_addr+ DEF_ONE+DEF_TWO+DEF_THREE+DEF_FOUR+DEF_FIVE,DEF_SIX}, //six
    {26,sum_addr+ DEF_ONE+DEF_TWO+DEF_THREE+DEF_FOUR+DEF_FIVE+DEF_SIX,DEF_SEVEN}, //seven
    {27,sum_addr+ DEF_ONE+DEF_TWO+DEF_THREE+DEF_FOUR+DEF_FIVE+DEF_SIX+DEF_SEVEN,DEF_EIGHT}, //eight
    {28,sum_addr+ DEF_ONE+DEF_TWO+DEF_THREE+DEF_FOUR+DEF_FIVE+DEF_SIX+DEF_SEVEN+DEF_EIGHT,DEF_NINE}, //nine
    {29,sum_addr+ DEF_ONE+DEF_TWO+DEF_THREE+DEF_FOUR+DEF_FIVE+DEF_SIX+DEF_SEVEN+DEF_EIGHT+DEF_NINE,DEF_TEN}, //ten
    {30,sum_addr+ DEF_ONE+DEF_TWO+DEF_THREE+DEF_FOUR+DEF_FIVE+DEF_SIX+DEF_SEVEN+DEF_EIGHT+DEF_NINE+DEF_TEN,DEF_TWENTY}, //twenty	

};	

/********************************************************************************
function:	
				Interrupt signal processing(ASR and Audio Player)
********************************************************************************/
void ProcessInt()
{
	UBYTE nAsrResCount = 0;		
	ucRegVal = LD_ReadReg(0x2B);
	ucHighInt = LD_ReadReg(0x29); // interrupt enable flag
	ucLowInt = LD_ReadReg(0x02); // interrupt enable flag
	LD_WriteReg(0x29, 0);// interrupt disenable
	LD_WriteReg(0x02, 0);// interrupt disenable
	if(nLD_Mode == LD_MODE_ASR_RUN) {
		LOG("---------------ASR---------------\r\n");
		//The interruption caused by speech recognition
		//(There is sound input, and there is interruption whether the recognition is successful or failed)
		if((ucRegVal & 0x10) && LD_ReadReg(0xb2)==0x21 && LD_ReadReg(0xbf)==0x35) {
			nAsrResCount = LD_ReadReg(0xba);
			if(nAsrResCount>0 && nAsrResCount<=4) {
				LOG("ASR SUCCESSFUL \r\n");
				nAsrStatus = LD_ASR_FOUNDOK;
			}
			else {
				LOG("ASR UNSUCCESSFUL \r\n");
				nAsrStatus = LD_ASR_FOUNDZERO;
			}	
		}
		else {
			LOG("No ASR \r\n");
			nAsrStatus = LD_ASR_FOUNDZERO;
		}			
		LD_WriteReg(0x2b, 0);
    LD_WriteReg(0x1c, 0);
		return;
	}	
	LOG("--------------PLAY MP3--------------\r\n");
	// Play MP3 to produce 3 kinkd of intterupt
	// A. play over
	// B. data send over
	// C. Data will be used up and sent
	if(LD_ReadReg(0xBA) & CAUSE_MP3_SONG_END) {
		// A. play over
		LD_WriteReg(0x2B, 0);
		LD_WriteReg(0xBA, 0);	
		LD_WriteReg(0xBC, 0);	
		LD_WriteReg(0x08, 1);
		LD_WriteReg(0x08, 0);
		LD_WriteReg(0x33, 0);
		LOG("play over \r\n");
		bMp3Play = 0;					// play status
		return;
	}
	if(nMp3Pos >= nMp3Size) {
		// B. data send over
		LD_WriteReg(0xBC, 0x01);//data voer
		LD_WriteReg(0x29, 0x10);
		return;	
	}
	// C. Data will be used up and sent	
	LD_ReloadMp3Data_from_flash();
//	LD_ReloadMp3Data();
	LD_WriteReg(0x29, ucHighInt); 
	LD_WriteReg(0x02, ucLowInt);	
}

/********************************************************************************
function:	
				Run ASR
********************************************************************************/
UBYTE LD_ASR(void)
{
	UBYTE i = 0;
	UBYTE asrflag = 0;
	for (i=0; i<5; i++) {						//run ASR try 5 times
		LD_Init_ASR();								//init ASR
		Driver_Delay_ms(100);
		if (LD_AsrAddKey() == 0) {		//Add fixed to LD3320
			LOG("ERROR!!! LD_AsrAddKey\r\n");
			LD_init();									//ERROR,Reset LD3320
			Driver_Delay_ms(50);	
			continue;
		}
		Driver_Delay_ms(10);
		if (LD_AsrRun() == 0) {				//start ASR
			LOG("ERROR!!! LD_AsrRun\r\n");
			LD_init();									//ERROR,Reset LD3320
			Driver_Delay_ms(50);
			continue;
		}
		asrflag = 1;
		break;						
	}	
	LOG("RunASR\r\n");
	return asrflag;
}

/********************************************************************************
function:	
				Start play
********************************************************************************/
void LD_play(UBYTE *mp3name)
{
	nMp3Pos = 0;
	bMp3Play = 1;
	nMp3name = mp3name;
	if(nMp3Pos >=  nMp3Size)
		return; 
	
	LD_ReloadMp3Data();

    LD_WriteReg(0xBA, 0x00);
	LD_WriteReg(0x17, 0x48);//activate DSP
	LD_WriteReg(0x33, 0x01);//play mp3
	LD_WriteReg(0x29, 0x04);//FIFO interrupt allowed
	LD_WriteReg(0x02, 0x01);//FIFO_DATA interrupt allowed 
	LD_WriteReg(0x85, 0x5A);
}

void PlaySoundFromFLASH(uint8_t nCode)
{

	if (nCode>=ITEM_COUNT)
		return;

	bMp3Play = 1;
	nMp3Pos =  pAsrItem[nCode].nSoundStart;
	nMp3Size =  nMp3Pos+pAsrItem[nCode].nSoundSize;
	
	if(nMp3Pos >=  nMp3Size)
	{
		bMp3Play = 0;
		return;
	}		
	LD_ReloadMp3Data_from_flash();

    LD_WriteReg(0xBA, 0x00);
	LD_WriteReg(0x17, 0x48);//activate DSP
	LD_WriteReg(0x33, 0x01);//play mp3
	LD_WriteReg(0x29, 0x04);//FIFO interrupt allowed
	LD_WriteReg(0x02, 0x01);//FIFO_DATA interrupt allowed 
	LD_WriteReg(0x85, 0x5A);	
}
/********************************************************************************
function:	
				Adjust headset volume
note:
				MAX=15 MIN=0
********************************************************************************/
void LD_Adjust_Volume(UBYTE val)
{
	val = ((15-val)&0x0f) << 2;
	LD_WriteReg(0x8e, val | 0xc1); 	//volume
	LD_WriteReg(0x87, 0x78);		//accept adjust
}

/********************************************************************************
function:	
				Get LD3320 return value
********************************************************************************/
UBYTE LD_GetResult(void)
{
	return LD_ReadReg(0xc5);
}

/********************************************************************************
function:	
				LD3320 hardware init
********************************************************************************/
void LD_init(void)
{
	RST_1;
	Driver_Delay_ms(100);
	RST_0;
	Driver_Delay_ms(100);
	RST_1;
	
	Driver_Delay_ms(100);
	CS_0;
	Driver_Delay_ms(100);
	CS_1;		
	Driver_Delay_ms(100);
	
//	LD_ReadReg(06);
//	LD_WriteReg(0x35, 0x33);
//	LD_WriteReg(0x1b, 0x55);
//	LD_WriteReg(0xb3, 0xaa);
//	LOG(" %x \n", LD_ReadReg(0x35));	
//	LOG(" %x \n", LD_ReadReg(0x1b));	
//	LOG(" %x \n", LD_ReadReg(0xb3));
//	
//	LOG(" %x \n", LD_ReadReg(0x06));
//	LOG(" %x \n", LD_ReadReg(0x06));
//	LOG(" %x \n", LD_ReadReg(0x35));
//	LOG(" %x \n", LD_ReadReg(0xb3));
}

/********************************************************************************
function:	
				Add ASR Keyword
********************************************************************************/
static UBYTE LD_AsrAddKey(void)
{
	UBYTE k, flag;
	UBYTE nAsrAddLength;
	#define DATE_A  10
	#define DATE_B 18	

	UBYTE  sRecog[DATE_A][DATE_B] = {		//add commond,use pinying
				"xiao xia", \
	 			"kong qi zhi liang",\
				"bao shi",\
				"qian jin",\
				"hou tui",\
				"zuo zhuan",\
				"you zhuan",\
				"yin yue",\
				"xia yi shou",\
				"shang yi shou",\
		
																	};	
	UBYTE  pCode[DATE_A] = {						//add commond code to do the commond
				CODE_RUN,	\
				CODE_AIR_QUALITY,	\
				CODE_PLAY_TIME,	\
				CODE_FORWARD,\
				CODE_BACKWARD,\
				CODE_TURNLEFT,\
				CODE_TURNRIGHT,\
				CODE_MUSIC,\
				CODE_NEXT_MUSIC,\
				CODE_PREVIOUS_MUSIC,\
															};	
	flag = 1;
	for(k=0; k<DATE_A; k++) {						//write data to LD3320	
		if(LD_Check_ASRBusyFlag() == 0) {
			flag = 0;
			break;
		}
		LD_WriteReg(0xc1, pCode[k]);
		LD_WriteReg(0xc3, 0);
		LD_WriteReg(0x08, 0x04);
		Driver_Delay_ms(1);
		LD_WriteReg(0x08, 0x00);
		Driver_Delay_ms(1);
		for (nAsrAddLength=0; nAsrAddLength<DATE_B; nAsrAddLength++) {
			if (sRecog[k][nAsrAddLength] == 0)
				break;
			LD_WriteReg(0x5, sRecog[k][nAsrAddLength]);
		}
		LD_WriteReg(0xb9, nAsrAddLength);
		LD_WriteReg(0xb2, 0xff);
		LD_WriteReg(0x37, 0x04);
	}	 
	return flag;
}

/********************************************************************************
function:	
				Reload mp3 data
********************************************************************************/
static void LD_ReloadMp3Data()
{
	UBYTE val;
	ucStatus = LD_ReadReg(0x06);
	while (!(ucStatus&MASK_FIFO_STATUS_AFULL) && (nMp3Pos<nMp3Size)) {
		//LOG("ucStatus is %x, write mp3 data: %x\r\n", ucStatus, val);
		val = nMp3name[nMp3Pos++];
		LD_WriteReg(0x01, val);
		ucStatus = LD_ReadReg(0x06);	
	}
}
/********************************************************************************/
static void LD_ReloadMp3Data_from_flash(void)
{
	UBYTE val;
	ucStatus = LD_ReadReg(0x06);
	while (!(ucStatus&MASK_FIFO_STATUS_AFULL) && (nMp3Pos<nMp3Size)) {
		//LOG("ucStatus is %x, write mp3 data: %x\r\n", ucStatus, val);
		W25QXX_Read(&val,nMp3Pos,1);					//ӵ100ַʼ,SIZEֽ
		//spiȡ
		nMp3Pos++;
		LD_WriteReg(0x01, val);
		ucStatus = LD_ReadReg(0x06);	
	}
}
/********************************************************************************
function:	
				Write data to LD3320 reg
********************************************************************************/
static void LD_WriteReg(UBYTE data1,UBYTE data2)
{
	CS_0;
	WR_0;
	spi_send_byte(0x04);
	spi_send_byte(data1);
	spi_send_byte(data2);
	CS_1;
}

/********************************************************************************
function:	
				Read data from LD3320 reg
********************************************************************************/
static UBYTE LD_ReadReg(UBYTE reg_add)
{
	UBYTE i;
	CS_0;
	WR_0;
	spi_send_byte(0x05);
	spi_send_byte(reg_add);
	i = spi_send_byte(0x00);
	CS_1;
	return(i);
}

/********************************************************************************
function:	
				Begin to ASR
********************************************************************************/
static UBYTE LD_AsrRun(void)
{
	LD_WriteReg(0x35, MIC_VOL);
	LD_WriteReg(0x1C, 0x09);
	LD_WriteReg(0xBD, 0x20);
	LD_WriteReg(0x08, 0x01);
	Driver_Delay_ms(20);
	LD_WriteReg(0x08, 0x00);
	Driver_Delay_ms(20);

	if(LD_Check_ASRBusyFlag() == 0)
		return 0;
	
	LD_WriteReg(0xB2, 0xff);	
	LD_WriteReg(0x37, 0x06);
	LD_WriteReg(0x37, 0x06);
	Driver_Delay_ms(20);
	LD_WriteReg(0x1C, 0x0b);
	LD_WriteReg(0x29, 0x10);
	LD_WriteReg(0xBD, 0x00);   
	return 1;
}

/********************************************************************************
function:	
				Check ASR state
********************************************************************************/
static UBYTE LD_Check_ASRBusyFlag(void)
{
	UBYTE j;
	UBYTE flag = 0;
	for (j=0; j<10; j++) {
		if (LD_ReadReg(0xb2) == 0x21) {			
			flag = 1;
			break;
		}
		Driver_Delay_ms(10);
		//LOG("ERROR!!! ASR Busy\r\n");
	}
	return flag;
}

/********************************************************************************
function:	
				Common init
********************************************************************************/
static void LD_Init_Common(void)
{
	bMp3Play = 0;
	
	LD_ReadReg(0x06);  
	LD_WriteReg(0x17, 0x35); 
	Driver_Delay_ms(20);
	LD_ReadReg(0x06);  

	LD_WriteReg(0x89, 0x03);  
	Driver_Delay_ms(20);
	LD_WriteReg(0xCF, 0x43);   
	Driver_Delay_ms(20);
	LD_WriteReg(0xCB, 0x02);
	
	/*PLL setting*/
	LD_WriteReg(0x11, LD_PLL_11);       
	if (nLD_Mode == LD_MODE_MP3) {
		LD_WriteReg(0x1E, 0x00); 
		LD_WriteReg(0x19, LD_PLL_MP3_19);   
		LD_WriteReg(0x1B, LD_PLL_MP3_1B);   
		LD_WriteReg(0x1D, LD_PLL_MP3_1D);
	}
	else {
		LD_WriteReg(0x1E,0x00);
		LD_WriteReg(0x19, LD_PLL_ASR_19); 
		LD_WriteReg(0x1B, LD_PLL_ASR_1B);		
		LD_WriteReg(0x1D, LD_PLL_ASR_1D);
	}
	Driver_Delay_ms(20);
	
	LD_WriteReg(0xCD, 0x04);
	LD_WriteReg(0x17, 0x4c); 
	Driver_Delay_ms(20);
	LD_WriteReg(0xB9, 0x00);
	LD_WriteReg(0xCF, 0x4F); 
	LD_WriteReg(0x6F, 0xFF); 
}

/********************************************************************************
function:	
				ASR init
********************************************************************************/
static void LD_Init_ASR(void)
{
	nLD_Mode=LD_MODE_ASR_RUN;
	LD_Init_Common();

	LD_WriteReg(0xBD, 0x00);
	LD_WriteReg(0x17, 0x48);	
	Driver_Delay_ms(20);
	
	LD_WriteReg(0x3C, 0x80);    
	LD_WriteReg(0x3E, 0x07);
	LD_WriteReg(0x38, 0xff);    
	LD_WriteReg(0x3A, 0x07);
	Driver_Delay_ms(20);
	
	LD_WriteReg(0x40, 0);          
	LD_WriteReg(0x42, 8);
	LD_WriteReg(0x44, 0);    
	LD_WriteReg(0x46, 8); 
	Driver_Delay_ms(20);
}

/********************************************************************************
function:	
				MP3 player init
********************************************************************************/
void LD_Init_MP3()
{
	nLD_Mode = LD_MODE_MP3;
	LD_Init_Common();

	LD_WriteReg(0xBD, 0x02);
	LD_WriteReg(0x17, 0x48);
	Driver_Delay_ms(20);

	LD_WriteReg(0x85, 0x52); 
	LD_WriteReg(0x8F, 0x00);  
	LD_WriteReg(0x81, 0x00);
	LD_WriteReg(0x83, 0x00);
	LD_WriteReg(0x8E, 0xff);
	LD_WriteReg(0x8D, 0xff);
	Driver_Delay_ms(20);
	LD_WriteReg(0x87, 0xff);
	LD_WriteReg(0x89, 0xff);
	Driver_Delay_ms(20);
	LD_WriteReg(0x22, 0x00);    
	LD_WriteReg(0x23, 0x00);
	LD_WriteReg(0x20, 0xef);    
	LD_WriteReg(0x21, 0x07);
	LD_WriteReg(0x24, 0x77);          
	LD_WriteReg(0x25, 0x03);
	LD_WriteReg(0x26, 0xbb);    
	LD_WriteReg(0x27, 0x01); 
}
void Force_Clear_Play(void)
{
		LD_WriteReg(0x2B, 0);
		LD_WriteReg(0xBA, 0);	
		LD_WriteReg(0xBC, 0);	
		LD_WriteReg(0x08, 1);
		LD_WriteReg(0x08, 0);
		LD_WriteReg(0x33, 0);
		LOG("play over \r\n");
		bMp3Play = 0;	
}
