#include "hal_adc.h"

#define TOTAL_CONVERT_CH_NUM        2	                                                                                     //要转换的通道数
#define EVERY_CH_CONVERT_TIMES      100                                                                                     //单个通道转换次数
#define ADC1_BUF_SIZE               (TOTAL_CONVERT_CH_NUM*EVERY_CH_CONVERT_TIMES)                                          //DMA要传输的数据总大小 5*100

u16 ADCConvertedValue_Tab[ADC1_BUF_SIZE] = {0};

void Adc_Gpio_Init(void)
{
	GPIO_InitTypeDef GPIO_InitStructure;
	
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC, ENABLE);
	
	//PC00---ADC12_IN10
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0; //电机电流
	GPIO_InitStructure.GPIO_Mode  = GPIO_Mode_AIN;
	GPIO_Init(GPIOC, &GPIO_InitStructure);
}

void Adc_Init(void)
{
	ADC_InitTypeDef ADC_InitStructure;
	DMA_InitTypeDef DMA_InitStructure;
	
	
	RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, ENABLE);    //打开DMA1时钟
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);  //打开ADC1时钟
	
	RCC_ADCCLKConfig(RCC_PCLK2_Div6);  //72/6 = 12MHz,  ADC1时钟MAX=14MHz
	
	/* DMA1 channel1 configuration ----------------------------------------------*/
	DMA_DeInit(DMA1_Channel1);
	DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)&(ADC1->DR); //ADC1->DR
	DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)&ADCConvertedValue_Tab;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
	DMA_InitStructure.DMA_BufferSize = ADC1_BUF_SIZE;//DMA要传输的数据总大小，本次传输50个数据
	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;//外设地址固定
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable; //内存地址变，接收一次数据后，内存地址是否后移，用来采集多个数据
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
	DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;//循环传输，适合ADC模式
	DMA_InitStructure.DMA_Priority = DMA_Priority_High;
	DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
	DMA_Init(DMA1_Channel1, &DMA_InitStructure);
  
	/* Enable DMA1 channel1 */
	DMA_Cmd(DMA1_Channel1, ENABLE);

	/* ADC1 configuration ------------------------------------------------------*/
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
	ADC_InitStructure.ADC_ScanConvMode = ENABLE;
	ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = TOTAL_CONVERT_CH_NUM;//要转换的通道数
	ADC_Init(ADC1, &ADC_InitStructure);
	
	ADC_TempSensorVrefintCmd(ENABLE); //开启内部温度传感器	
	
	/* ADC1 regular channel.11/13/14/15 configuration */ 
	ADC_RegularChannelConfig(ADC1, ADC_Channel_10, 1, ADC_SampleTime_239Cycles5);
	ADC_RegularChannelConfig(ADC1, ADC_Channel_16, 2, ADC_SampleTime_239Cycles5 );	//ADC1,ADC通道3,第一个转换,采样时间为239.5周期	  			    
	/* Enable ADC1 DMA */
	ADC_DMACmd(ADC1, ENABLE);

	/* Enable ADC1 */
	ADC_Cmd(ADC1, ENABLE);

	/* Enable ADC1 reset calibration register */   
	ADC_ResetCalibration(ADC1);
	/* Check the end of ADC1 reset calibration register */
	while(ADC_GetResetCalibrationStatus(ADC1));

	/* Start ADC1 calibration */  //校准
	ADC_StartCalibration(ADC1);
	/* Check the end of ADC1 calibration */
	while(ADC_GetCalibrationStatus(ADC1));

	/* Start ADC1 Software Conversion */ //软件触发，非外部触发
	ADC_SoftwareStartConvCmd(ADC1, ENABLE);
}

void Hal_Adc_Init(void)//初始化ADC1
{
	Adc_Gpio_Init();
	Adc_Init();
	LOG("\r\nADCInit\r\n");
}

//uint16_t
uint32_t AdcValFilter(uint8_t ChannelNum)//采集ADC通道数据
{
  uint32_t temp = 0;
	for(char i=0; i<EVERY_CH_CONVERT_TIMES; i++)
	{
		temp  += (uint32_t)(ADCConvertedValue_Tab[TOTAL_CONVERT_CH_NUM*i + ChannelNum] & 0x0FFF);
		//LOG("ADCTab%.2d[CHNUM%d]:%d\r\n", i, ChannelNum, (ADCConvertedValue_Tab[TOTAL_CONVERT_CH_NUM*i + ChannelNum] & 0x0FFF));
	}
	
	return (uint32_t)(temp);
}

void AdcRead(void)//采集模拟量
{
	LOG("adc: bat    %d\n"	, AdcValFilter(0)/EVERY_CH_CONVERT_TIMES); 
	LOG("adc: temperate %d\n"	, AdcValFilter(1)/EVERY_CH_CONVERT_TIMES); 
}  
//得到温度值
//返回值:温度值(扩大了100倍,单位:℃.)
short Get_Temprate(void)	//获取内部温度传感器温度值
{
	short result;
 	double temperate;
	temperate=(float)AdcValFilter(1)/EVERY_CH_CONVERT_TIMES*(3.3/4096);		//电压值 
	temperate=(1.42-temperate)/0.00435+25;	//转换为温度值 	 
	result=temperate;					//扩大100倍.
	return result;
}
