#include "hal_air_quality_sensor.h"
#include "delay.h"
static void USART3_Configuration(void);
//static void IAR_Send_Initiative_Auto_Update(void);
static void IAR_Send_Initiative_Passive_Update(void);
static void IAR_Send_Request_Mix_Data(void);
void IAR_Send_Request_Close_Led(void);
void IAR_Send_Request_Open_Led(void);
void IAR_Parse_Data(void);

uint8_t CalCheckSum(uint8_t *data, uint16_t len);
uint8_t expect_receive_length=0;
uint8_t iaq_fsm_sta=0;
DlyTypeDef IAQ_FSM_Dly;

#define SCALE 1000

//串口3队列定义
#define UART3BuffSize 100
u8 	UART3SendBuff[UART3BuffSize];		//发送数据
u8 	UART3ReceBuff[UART3BuffSize];		//接收数据？
u16 UART3ReceIn = 0;//接收状态标记数据位	 
u8  UART3ReceFullFlag = 0;//接收完数据标志位

//static int maximum_range=0;
//static float air_quality_co=0.0;
//static float air_quality_voc=0.0;
//static float air_quality_tvoc=0.0;
static float air_quality_ppm=0.0;
//static float air_quality_mg =0.0;
static float air_temperature_c=0.0;
static float air_humidity_rh =0.0;

float Get_IAR_PPM(void)
{
	return air_quality_ppm;
}
float Get_IAR_Temperature(void)
{
	return air_temperature_c;
}
float Get_IAR_Humidity(void)
{
	return air_humidity_rh;	
}
void IAR_Init(void)
{
	USART3_Configuration();
	//	IAR_Send_Initiative_Auto_Update();//不需要使用被动上报
	IAR_Send_Initiative_Passive_Update();
	IAR_Send_Request_Open_Led();
}
void IAR_Process(void)
{
	switch(iaq_fsm_sta)
	{
		case 0:
			if(IAQ_FSM_Dly.flag==1)break;
			IAR_Send_Request_Mix_Data();
			IAQ_FSM_Dly.flag =1;
			IAQ_FSM_Dly.src  = 300;
			break;
		case 1:
			IAR_Parse_Data();
			break;
	}
//	IAR_Send_Request_Close_Led();
//	Timer_Delay_Ms(2000);
//	IAR_Send_Request_Open_Led();
//	Timer_Delay_Ms(2000);
}
void IAR_Parse_Data(void)
{
	if(UART3ReceFullFlag)
	{
		if(expect_receive_length<2)return;
		uint8_t buff[13]={0};
		memcpy(buff,UART3ReceBuff,expect_receive_length);
		if(buff[expect_receive_length-1]==CalCheckSum(buff,expect_receive_length-1))
		{
//			air_quality_mg    =(float)(buff[2]<<8|buff[3])/SCALE;
			air_quality_ppm   =(float)(buff[6]<<8|buff[7])/SCALE;
			air_temperature_c =(float)(buff[8]<<8|buff[9])/100;
			air_humidity_rh   =(float)(buff[8]<<8|buff[11])/100;
			
//			air_quality_tvoc=air_quality_ppm*0.35;
//			air_quality_voc=air_quality_ppm*0.85;
//			air_quality_co=air_quality_ppm*0.7;
		}
		UART3ReceIn=0;
		iaq_fsm_sta=0;

	}
}
/*  0.1ms   */
void IRQ_FSM_TIM_CR(void)
{
	if(IAQ_FSM_Dly.flag==1)
	{
		(IAQ_FSM_Dly.count<IAQ_FSM_Dly.src)?(IAQ_FSM_Dly.count++):(IAQ_FSM_Dly.flag=0,iaq_fsm_sta=1);
  }
}
void USART3_Configuration(void)
{
 //GPIO端口设置
	GPIO_InitTypeDef GPIO_InitStructure;
	USART_InitTypeDef USART_InitStructure;
	NVIC_InitTypeDef NVIC_InitStructure;

	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOC|RCC_APB2Periph_GPIOD|RCC_APB2Periph_AFIO, ENABLE); //remap时钟|RCC_APB2Periph_AFIO //开启GPIOB时钟
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);//这里要分开打开 //USART3时钟  来自APB1

	//GPIO_PartialRemap_USART3 部分重映射  GPIOC_10  GPIOC_11
	GPIO_PinRemapConfig(GPIO_PartialRemap_USART3,ENABLE);  
	//USART3_TX   GPIOC.10
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10; //
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;	//复用推挽输出
	GPIO_Init(GPIOC, &GPIO_InitStructure);//初始化

	//USART3_RX	  GPIOC.11初始化
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;//
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;//浮空输入
	GPIO_Init(GPIOC, &GPIO_InitStructure);//初始 
  //Usart3 NVIC 配置
	NVIC_InitStructure.NVIC_IRQChannel = USART3_IRQn;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority=1 ;//抢占优先级3
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 2;		//子优先级3
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;			//IRQ通道使能
	NVIC_Init(&NVIC_InitStructure);	//根据指定的参数初始化VIC寄存器

	//USART 初始化设置

	USART_InitStructure.USART_BaudRate = 9600;//串口波特率
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;//字长为8位数据格式
	USART_InitStructure.USART_StopBits = USART_StopBits_1;//一个停止位
	USART_InitStructure.USART_Parity = USART_Parity_No;//无奇偶校验位
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;//无硬件数据流控制
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;	//收发模式

	USART_Init(USART3, &USART_InitStructure); //初始化串口3
	USART_ITConfig(USART3, USART_IT_RXNE, ENABLE);//开启串口接受中断
	USART_Cmd(USART3, ENABLE);                    //使能串口3 
}
uint8_t CalCheckSum(uint8_t *data, uint16_t len)
{
    uint8_t checksum = 0;
    uint8_t i = 0;

    for (i = 0; i < len; i++)
    {
        checksum += *data++;
    }
//	checksum &= 0x00ff;
//	checksum += 1;
	checksum = 0xff -checksum ;
    return checksum;
}
void IAR_Send_Initiative_Auto_Update(void)
{
	uint8_t buf[9]= {0xff,0x01,0x78,0x40,0x00,0x00,0x00,0x00,0x47};
	USART_SendNData(USART3,(char *)buf,sizeof(buf));
	expect_receive_length=9;
}
void IAR_Send_Initiative_Passive_Update(void)
{
	uint8_t buf[9]= {0xff,0x01,0x78,0x41,0x00,0x00,0x00,0x00,0x46};
	USART_SendNData(USART3,(char *)buf,sizeof(buf));
	expect_receive_length=0;
}
void IAR_Send_Request_Mix_Data(void)
{
	uint8_t buf[9]= {0xff,0x01 ,0x87, 0x00,0x00, 0x00 ,0x00, 0x00, 0x78};
	USART_SendNData(USART3,(char *)buf,sizeof(buf));
	expect_receive_length=13;
}
void IAR_Send_Request_Close_Led(void)
{
	uint8_t buf[9]= {0xFF , 0x01,  0x88,  0x00,  0x00,  0x00,  0x00,  0x00,  0x77};
	USART_SendNData(USART3,(char *)buf,sizeof(buf));
	expect_receive_length=2;
}
void IAR_Send_Request_Open_Led(void)
{
	uint8_t buf[9]= {0xFF , 0x01,  0x89,  0x00,  0x00,  0x00,  0x00,  0x00,  0x76};
	USART_SendNData(USART3,(char *)buf,sizeof(buf));
	expect_receive_length=2;
}
void USART_SendNData(USART_TypeDef* USARTx,char *data,int len)//UART直接发送
{
  char i;
	
  for(i=0;i<len;i++)
  {
    while(USART_GetFlagStatus(USARTx,USART_FLAG_TC) == RESET);
    USART_SendData(USARTx,data[i]);
  }
}

void Air_Quality_Recv_Irq(void)
{
	u8 Res;//数据暂存
	if(USART_GetITStatus(USART3, USART_IT_RXNE) != RESET)  //接收中断
	{
		Res =USART_ReceiveData(USART3);	//读取接收到的数据 USART_ReceiveData
			//USART_SendData(USART3,Res);//将读取到的数据通过串口1发送
		if(UART3ReceIn>=UART3BuffSize)return;
		switch(UART3ReceIn)//读取接收到的数据有几位 每一位对应的数据协议校验
		{
			case 0:
				if(Res==0XFF)
					UART3ReceBuff[UART3ReceIn++] = Res;
				else
					UART3ReceIn = 0;
				break;
//			case 8:
//				if(Res==0xFF)
//					UART3ReceBuff[UART3ReceIn++] = Res;
//				else
//					UART3ReceIn = 0;
//				break;
			default:
				UART3ReceBuff[UART3ReceIn++] = Res;
				break;
		}
 
		if(UART3ReceIn >= expect_receive_length)
		{
			UART3ReceFullFlag = 1;	//数据完整接受完 
		}
		USART_ClearITPendingBit(USART3, USART_IT_RXNE);//清除相对应的中断位   清除中断预处理位USART_ClearITPendingBit左移八位是USART_ClearFlag  
	}
	else if(USART_GetITStatus(USART3, USART_IT_TXE) != RESET)	// 发送中断  USART_GetITStatus
	{
		USART_ClearITPendingBit(USART3, USART_IT_TXE);					// clear interrupt  清除中断预处理位
	}	

}
