#include "hal_rtc.h"
#include "delay.h"
#include <stdlib.h>

/*全局变量*/
_calendar_obj calendar;//时钟结构体 
SysFlagStructTypeDef 		SysFlag;

void ExtiRtcCfgDisable(void);

static void RTC_NVIC_Config(void)
{	
  NVIC_InitTypeDef NVIC_InitStructure;
	
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_1);
	
	#if RTC_IT_SEC_EN
	NVIC_InitStructure.NVIC_IRQChannel = RTC_IRQn;		//RTC全局中断
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;	//先占优先级1位,从优先级3位
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;	//先占优先级0位,从优先级4位
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;		//使能该通道中断
	NVIC_Init(&NVIC_InitStructure);		//根据NVIC_InitStruct中指定的参数初始化外设NVIC寄存器
	#endif
	
	NVIC_InitStructure.NVIC_IRQChannel = RTCAlarm_IRQn;		//RTC闹钟中断
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;	//先占优先级1位,从优先级3位
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;	//先占优先级0位,从优先级4位
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;		//使能该通道中断
	NVIC_Init(&NVIC_InitStructure);	
}

/**
  * @brief  Configures EXTI Lines.
  * @param  None
  * @retval None
  */
void ExtiRtcCfg(void)
{
  EXTI_InitTypeDef EXTI_InitStructure;

  /* Configure EXTI Line17(RTC Alarm) to generate an interrupt on rising edge */
  EXTI_ClearITPendingBit(EXTI_Line17);
  EXTI_InitStructure.EXTI_Line = EXTI_Line17;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  EXTI_Init(&EXTI_InitStructure);
}

/*
功能:关闭EXTI_Line17
*/
void ExtiRtcCfgDisable(void)
{
  EXTI_InitTypeDef EXTI_InitStructure;

  /* Configure EXTI Line17(RTC Alarm) to generate an interrupt on rising edge */
  EXTI_ClearITPendingBit(EXTI_Line17);
  EXTI_InitStructure.EXTI_Line = EXTI_Line17;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising;
  EXTI_InitStructure.EXTI_LineCmd = DISABLE;
  EXTI_Init(&EXTI_InitStructure);
}

//实时时钟配置
//初始化RTC时钟,同时检测时钟是否工作正常
//BKP->DR1用于保存是否第一次配置的设置
//返回0:正常
// 其他:错误代码
u8 RTC_Init(void)
{
	ExtiRtcCfg(); /*Configure EXTI Line17(RTC Alarm) to generate an interrupt on rising edge*/
	
	RTC_NVIC_Config(); /*RCT中断分组设置*/
	
//检查是不是第一次配置时钟
	u8 temp=0;
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);	//使能PWR和BKP外设时钟   
	PWR_BackupAccessCmd(ENABLE);	//使能后备寄存器访问  
	if (BKP_ReadBackupRegister(BKP_DR1) != 0x5050)		//从指定的后备寄存器中读出数据:读出了与写入的指定数据不相乎
	{	 			
		BKP_DeInit();	//复位备份区域 	
		RCC_LSEConfig(RCC_LSE_ON);	//设置外部低速晶振(LSE),使用外设低速晶振
		while (RCC_GetFlagStatus(RCC_FLAG_LSERDY) == RESET&&temp<250)	//检查指定的RCC标志位设置与否,等待低速晶振就绪
		{
			temp++;
			Timer_Delay_Ms(10);
		}
		if(temp>=250)return 1;//初始化时钟失败,晶振有问题	    
		RCC_RTCCLKConfig(RCC_RTCCLKSource_LSE);		//设置RTC时钟(RTCCLK),选择LSE作为RTC时钟    
		RCC_RTCCLKCmd(ENABLE);	//使能RTC时钟  
		RTC_WaitForLastTask();	//等待最近一次对RTC寄存器的写操作完成
		
		RTC_WaitForSynchro();		//等待RTC寄存器同步  
		RTC_ITConfig(RTC_IT_SEC, ENABLE);		//使能RTC秒中断
		RTC_WaitForLastTask();	//等待最近一次对RTC寄存器的写操作完成
		
		RTC_ITConfig(RTC_IT_ALR, ENABLE);	/*使能闹钟中断*/
		RTC_WaitForLastTask(); /*等待写入完成*/
		
		RTC_EnterConfigMode();/// 允许配置	
		RTC_SetPrescaler(32767); //设置RTC预分频的值
		RTC_WaitForLastTask();	//等待最近一次对RTC寄存器的写操作完成
		RTC_Set(2021,9,20,13,50,55);  //设置时间	
		RTC_ExitConfigMode(); //退出配置模式  
		BKP_WriteBackupRegister(BKP_DR1, 0X5050);	//向指定的后备寄存器中写入用户程序数据
		}
	else//系统继续计时
	{
		#if RTC_IT_SEC_EN
		RTC_WaitForSynchro();	//等待最近一次对RTC寄存器的写操作完成
		RTC_ITConfig(RTC_IT_SEC, ENABLE);	//使能RTC秒中断
		RTC_WaitForLastTask();	//等待最近一次对RTC寄存器的写操作完成
		#endif
		
		RTC_WaitForSynchro();  /*等待RTC寄存器同步*/
		RTC_ITConfig(RTC_IT_ALR, ENABLE);	/*使能闹钟中断*/
		RTC_WaitForLastTask(); /*等待写入完成*/
	}
	RTC_NVIC_Config();//RCT中断分组设置		    				     
	RTC_Get();//更新时间			
	
	LOG(".RTC inited\r\n");
	
	return 0; //ok
}

/*RTC时钟中断
功能:每秒更新RTC,时间直接存放在全局日历变量calendar中
注:中断每秒触发一次    
*/
#if RTC_IT_SEC_EN
void RTC_IRQHandler(void)
{		 
	if (RTC_GetITStatus(RTC_IT_SEC) != RESET)//秒钟中断
	{							
		RTC_Get();//更新时间
		RTC_WaitForLastTask();
//		ShowRTCTime(calendar);
		RTC_ClearITPendingBit(RTC_IT_SEC);	
		RTC_WaitForLastTask();
 	} 				  								 
}
#endif

/**
  * @brief  This function handles RTCAlarm_IRQHandler interrupt request.
  * @param  None
  * @retval : None
  */

void RTC_Alarm_IRQHandler(void)
{
  if(RTC_GetITStatus(RTC_IT_ALR) != RESET)
  {
    /* Clear EXTI line17 pending bit */
    EXTI_ClearITPendingBit(EXTI_Line17);

    /* Check if the Wake-Up flag is set */
    if(PWR_GetFlagStatus(PWR_FLAG_WU) != RESET)
    {
      /* Clear Wake Up flag */
      PWR_ClearFlag(PWR_FLAG_WU);
			SysFlag.rtcWkupINT = 1;
    }
    RTC_WaitForLastTask();   
    RTC_ClearITPendingBit(RTC_IT_ALR);
    RTC_WaitForLastTask();
  }
}


//判断是否是闰年函数
//月份   1  2  3  4  5  6  7  8  9  10 11 12
//闰年   31 29 31 30 31 30 31 31 30 31 30 31
//非闰年 31 28 31 30 31 30 31 31 30 31 30 31
//输入:年份
//输出:该年份是不是闰年.1,是.0,不是
u8 Is_Leap_Year(u16 year)
{			  
	if(year%4==0) //必须能被4整除
	{ 
		if(year%100==0) 
		{ 
			if(year%400==0)return 1;//如果以00结尾,还要能被400整除 	   
			else return 0;   
		}else return 1;   
	}else return 0;	
}	 			   
//设置时钟
//把输入的时钟转换为秒钟
//以1970年1月1日为基准
//1970~2099年为合法年份
//返回值:0,成功;其他:错误代码.
//月份数据表											 
const u8 table_week[12]={0,3,3,6,1,4,6,2,5,0,3,5}; //月修正数据表	  
const u8 mon_table[12]={31,28,31,30,31,30,31,31,30,31,30,31};//平年的月份日期表
u8 RTC_Set(u16 syear,u8 smon,u8 sday,u8 hour,u8 min,u8 sec)
{
	u16 t;
	u32 seccount=0;
	
	if(syear<1970||syear>2099) return 1;/*年错误*/
	/*---------lmj add 时间参数检查-----------------------------------*/
	if((smon==0) || (smon>12)) return 2;/*月错误*/
	if((sday==0) || (sday>31)) return 3;/*天错误*/
	if(hour>23) return 4;/*小时错误*/
	if(min>59)  return 5;/*分钟错误*/
	if(sec>59)  return 6;/*秒钟错误*/
	/*----------------------------------------------------------------*/
	
	for(t=1970;t<syear;t++)	//把所有年份的秒钟相加
	{
		if(Is_Leap_Year(t))seccount+=31622400;//闰年的秒钟数
		else seccount+=31536000;			  //平年的秒钟数
	}
	smon-=1;
	for(t=0;t<smon;t++)	   //把前面月份的秒钟数相加
	{
		seccount+=(u32)mon_table[t]*86400;//月份秒钟数相加
		if(Is_Leap_Year(syear)&&t==1)seccount+=86400;//闰年2月份增加一天的秒钟数	   
	}
	seccount+=(u32)(sday-1)*86400;//把前面日期的秒钟数相加 
	seccount+=(u32)hour*3600;//小时秒钟数
  seccount+=(u32)min*60;	 //分钟秒钟数
	seccount+=sec;//最后的秒钟加上去
  /*LOG("seccount%d\r\n", seccount);*/
	
	RCC_APB1PeriphClockCmd(RCC_APB1Periph_PWR | RCC_APB1Periph_BKP, ENABLE);	//使能PWR和BKP外设时钟  
	PWR_BackupAccessCmd(ENABLE);	//使能RTC和后备寄存器访问 
	RTC_SetCounter(seccount);	//设置RTC计数器的值

	RTC_WaitForLastTask();	//等待最近一次对RTC寄存器的写操作完成  	
	
	return 0;	    
}

/*
得到当前RTC时间
返回值:0,成功;其他:错误代码.
注:RTC秒中断每秒更新一次RTC数据
修改记录:
1、使用Sec2UTC函数处理19.3.24
*/
u8 RTC_Get(void)
{
	TimeMsgTypeDef tmputc;
	u32 tmpsec;
	
	tmpsec = RTC_GetCounter();
	Sec2UTC(tmpsec, &tmputc);	
	calendar.w_year  = tmputc.year;
	calendar.w_month = tmputc.month;
	calendar.w_date  = tmputc.day;
	calendar.hour    = tmputc.hour;
	calendar.min     = tmputc.min;
	calendar.sec     = tmputc.sec;
	calendar.week    = RTC_Get_Week(calendar.w_year, calendar.w_month, calendar.w_date);

	return 0;
}
#if 0 // 
u8 RTC_Get(void)
{
	static u16 daycnt=0;
	u32 timecount=0; 
	u32 temp=0;
	u16 temp1=0;	  
    
	timecount=RTC_GetCounter();	 
 	temp=timecount/86400;   //得到天数(秒钟数对应的)
	if(daycnt!=temp)//超过一天了
	{	  
		daycnt=temp;
		temp1=1970;	//从1970年开始
		while(temp>=365)
		{				 
			if(Is_Leap_Year(temp1))//是闰年
			{
				if(temp>=366)temp-=366;//闰年的秒钟数
				else {temp1++;break;}  
			}
			else temp-=365;	  //平年 
			temp1++;  
		}   
		calendar.w_year=temp1;//得到年份
		temp1=0;
		while(temp>=28)//超过了一个月
		{
			if(Is_Leap_Year(calendar.w_year)&&temp1==1)//当年是不是闰年/2月份
			{
				if(temp>=29)temp-=29;//闰年的秒钟数
				else break; 
			}
			else 
			{
				if(temp>=mon_table[temp1])temp-=mon_table[temp1];//平年
				else break;
			}
			temp1++;  
		}
		calendar.w_month=temp1+1;	//得到月份
		calendar.w_date=temp+1;  	//得到日期 
	}
	temp=timecount%86400;     		//得到秒钟数   	   
	calendar.hour=temp/3600;     	//小时
	calendar.min=(temp%3600)/60; 	//分钟	
	calendar.sec=(temp%3600)%60; 	//秒钟
	calendar.week=RTC_Get_Week(calendar.w_year,calendar.w_month,calendar.w_date);//获取星期   
	
	return 0;
}
#endif


//获得现在是星期几
//功能描述:输入公历日期得到星期(只允许1901-2099年)
//输入参数：公历年月日 
//返回值：星期号																						 
u8 RTC_Get_Week(u16 year,u8 month,u8 day)
{	
	u16 temp2;
	u8 yearH,yearL;
	
	yearH=year/100;	yearL=year%100; 
	// 如果为21世纪,年份数加100  
	if (yearH>19)yearL+=100;
	// 所过闰年数只算1900年之后的  
	temp2=yearL+yearL/4;
	temp2=temp2%7; 
	temp2=temp2+day+table_week[month-1];
	if (yearL%4==0&&month<3)temp2--;
	return(temp2%7);
}			  


/*
设置RTC下次唤醒时间
*/
void SET_RTC_WKUP_TIME(u32 second)
{
	/*打开AEXTI Line17(RTC Alarm)*/
	ExtiRtcCfg();

	RTC_SetAlarm(RTC_GetCounter()+ second); //设置下次RTC唤醒时间
	RTC_WaitForLastTask();
}
	

void clsRTCAlarm(void)//清除RTC中断信息
{	
	ExtiRtcCfgDisable(); /*关闭EXTI Line17(RTC Alarm)*/	
	SysFlag.rtcWkupINT = 0;//清除RCT中断标志
}


/*
将时间转换成秒数
注:计算从1970年到当前时间所经历过的秒数
从1970年1月1日
1999.12.31 23:30:00, 2000.1.1 00:30:00  (3600秒)
摘自:https://blog.csdn.net/wang93IT/article/details/79744711
*/
uint64_t UTC2Sec(TimeMsgTypeDef utcs)
{
	if(0 >= (int)(utcs.month -= 2)) /**//* 1..12 -> 11,12,1..10 */
	{
		utcs.month += 12;  /**//* Puts Feb last since it has leap day */
	  utcs.year  -= 1;
	}
	
	return (((((uint64_t)(utcs.year/4 - utcs.year/100 + utcs.year/400 + 367*utcs.month/12 + utcs.day)+ utcs.year*365 - 719499)*24 
		     + utcs.hour)*60 /*now have hours*/
	       + utcs.min)*60  /*now have minutes*/
	       + utcs.sec);    /*finally seconds*/
}

//将秒数转换成UTC 19.3.23
//输入:uint64_t seconds
//输出:TimeMsgTypeDef* out_utc
//返回:0-成功 其它-失败
u8 Sec2UTC(uint64_t seconds, TimeMsgTypeDef* out_utc)  //TEST OK 19.3.24
{
	u32 tmpday = 0;	
  u16 daycnt = 0;	
	u16 tmpyear= 0;	
  u16 tmpmon = 0;  
	u32 tmpsec = 0;
     
 	tmpday = seconds/86400;   //得到天数(秒钟数对应的)	
	if(daycnt != tmpday)//超过一天了
	{	  
		daycnt = tmpday;
		tmpyear = 1970;	//从1970年开始
		while(tmpday >= 365)
		{				 
			if(Is_Leap_Year(tmpyear))//是闰年
			{
				if(tmpday >= 366) 
					tmpday -= 366;//闰年的秒钟数
				else       
				{			 
					tmpyear++; 			 
					break;			 
				}  
			}
			else 
				tmpday -= 365;	  //平年 
			tmpyear++;  
		} 		
		out_utc->year = tmpyear; //得到年份		
		
		while(tmpday >= 28)//超过了一个月
		{
			if(Is_Leap_Year(out_utc->year) && tmpmon == 1)//当年是不是闰年/2月份
			{
				if(tmpday >= 29) 
					tmpday -= 29;//闰年的月天数
				else 
					break; 
			}
			else 
			{
				if(tmpday >= mon_table[tmpmon]) 
					tmpday -= mon_table[tmpmon];//平年
				else 
					break;
			}
			tmpmon++;  
		}
		out_utc->month = tmpmon + 1; //得到月份
		out_utc->day = tmpday + 1; //得到日期 
	}
		
	tmpsec = seconds%86400; //得到不足24小时的秒钟数  19.3.23 
	out_utc->hour = tmpsec/3600;     	//小时 ***
	out_utc->min = (tmpsec%3600)/60; 	//分钟 ***
	out_utc->sec = (tmpsec%3600)%60; 	//秒钟 ***
	
	return 0;	
}

/*
功能:以秒为单位，比较两个时间是否在误差范围内。
输入：
	nmea_utc_time utc1:时间1
	nmea_utc_time utc2:时间2
  u32 deviationSec:偏差绝对秒值 

返回:
  0-两个时间在误差范围内
  1-超过误差范围
*/
u8 compTwoTimeVal(TimeMsgTypeDef utc1, TimeMsgTypeDef utc2, u32 deviationSec)
{
	int32_t tmp1 = 0;
	int32_t tmp2 = 0;
	
	tmp1 = UTC2Sec(utc1);
	//LOG("tmp1:%d\r\n", tmp1);
	tmp2 = UTC2Sec(utc2);
	//LOG("tmp2:%d\r\n", tmp2);
	
	if(abs(tmp1 - tmp2) >= deviationSec)
	{
		//LOG("compAbs:%d\r\n", abs(tmp1 - tmp2));
		return 1; /*时间绝对偏差大于等于deviationVal*/
	}

  return 0;	
}

/*
按照UTC格式打印时间
*/
void ShowUTC(TimeMsgTypeDef tt)
{
	#ifdef DEBUG_PRINTF
	LOG("%.4d-%.2d-%.2d %.2d:%.2d:%.2d\r\n", tt.year, tt.month, tt.day, tt.hour, tt.min, tt.sec);
	#endif
}

void ShowRTCTime(_calendar_obj rtc)
{
	#ifdef DEBUG_PRINTF
	RTC_Get();
	LOG("RTC>%.4d.%.2d.%.2d %.2d:%.2d:%.2d W%d\r\n", rtc.w_year, rtc.w_month, rtc.w_date, rtc.hour, rtc.min, rtc.sec, rtc.week);
	#endif
}

