#ifndef __HAL_RTC_H__
#define __HAL_RTC_H__
#ifdef __cplusplus
 extern "C" {
#endif
#include "global_param.h"
#include "hk32f10x.h"

#define RTC_IT_SEC_EN 0
//Mini STM32开发板
//RTC实时时钟 驱动代码			 
//正点原子@ALIENTEK
//2010/6/6


__packed typedef struct  
{										
	u32 year;	 //年
	u32 month; //月
	u32 day;	 //日
	u32 hour;  //时
	u32 min; 	 //分
	u32 sec; 	 //秒
}TimeMsgTypeDef; //UTC时间结构体定义 /*nmea_utc_time 此结构体达不到精度,会导致 UTC2Sec 函数计算错误*/

//时间结构体
typedef struct 
{
	vu8 hour;
	vu8 min;
	vu8 sec;			
	//公历日月年周
	vu16 w_year;
	vu8  w_month;
	vu8  w_date;
	vu8  week;		 
}_calendar_obj;					 
extern _calendar_obj calendar;	//日历结构体
extern u8 const mon_table[12];	//月份日期数据表


__packed typedef struct{
	u32 rtcWkupINT			:1;			   //0 RTC中断唤醒
	u32 Gsenr						:1;			   //1 LIS3DH唤醒
	u32 CarKey					:1;			   //2 钥匙唤醒
	u32 WorkTout				:1;			   //3 工作超时
	u32 gMsgTxType			:1;			   //4 回复数据类型 	0-BT, 1-2G
	u32 mdmRecvTimeout	:1;			   //5 M2G_接收超时
	u32 disAnSteal			:1;  	 	   //6 撤销防盗后开始计数标志
	u32 mdmUpdate0203		:1;   	   //7 0203状态包更新 0-不更新 1-更新
	u32 devComMode			:1;			   //8 设备通讯方式 0-BT, 1-2G
	u32 m6313LPMEn			:1;			   //9 m6313低功耗使能			0-不进入LPM 1-进入LPM
	u32 mdmWkupINT			:1;			   //10 m6313唤醒中断					
	u32 mdm0203SrcType  :1;        //11 发送0203包数据源
	u32 findOnEn        :1;        //12 寻车使能 0-禁止 1-允许
	u32 mdmFSMRunSta    :1;        //13 MDM_FSM运行状态    0-未运行 1-运行中  用于第一次上电开机 在mdmComFSM中处理      
	u32 getLastTx0203TimeEn     :1;//14 读取last0203SendTime使能     0-关闭 1-打开
	u32 judgLastTx0203TimeOutEn :1;//15 判断last0203SendTime超时使能 0-关闭 1-打开
	u32 sysWkupTimeoutEn        :1;//16 系统唤醒后超时使能           0-关闭 1-打开    系统被唤醒后延时打开补发0203功能
	u32 mdmTCPClosed	          :1;//17 TCP关闭状态  0-未关闭 1-被关闭 
	u32 RdySlp                  :1;//18 系统休眠    0-无效   1-准备休眠	
	u32 Res							        :32-19;	   //保留
}SysFlagStructTypeDef; 



u8 RTC_Init(void);        //初始化RTC,返回0,失败;1,成功;

u8 Is_Leap_Year(u16 year);//平年,闰年判断
u8 RTC_Get(void);         //更新时间   
u8 RTC_Get_Week(u16 year,u8 month,u8 day);
u8 RTC_Set(u16 syear,u8 smon,u8 sday,u8 hour,u8 min,u8 sec);//设置时间			 

#if RTC_IT_SEC_EN
void RTC_IRQHandler(void);
#endif

void SET_RTC_WKUP_TIME(u32 second);
void clsRTCAlarm(void);

u8   compTwoTimeVal(TimeMsgTypeDef utc1, TimeMsgTypeDef utc2, u32 deviationSec);
void ShowUTC(TimeMsgTypeDef tt);
void ShowRTCTime(_calendar_obj rtc);

uint64_t UTC2Sec(TimeMsgTypeDef utcs);
u8       Sec2UTC(uint64_t seconds, TimeMsgTypeDef* out_utc);

#ifdef __cplusplus
}
#endif
#include "global_param.h"
#endif
