#include "driving_wheel.h"
#include "hal_driving_wheel.h"
#include "delay.h"
#include "robot_ctrl.h"


void Wheel_Construct(HwWheel_t *const _this)
{
	Hal_Driving_Periph_Init();
	_this->config_dir[WHEEL_ID_A] = Hal_Wheel_Set_Dir_A;
	_this->config_enable[WHEEL_ID_A] = HwWheel_Config_Wheel_A;
	_this->p_tim_ccr[WHEEL_ID_A] = Hal_Wheel_GetCcr_A();
	_this->is_enable[WHEEL_ID_A] = DISABLE;
	_this->p_undir_encoder[WHEEL_ID_A] = Hal_Wheel_GetWheel_A_Addr_UndirEncoder();
	_this->p_dir_encoder[WHEEL_ID_A] = HwWheel_GetWheel_A_Addr_DirEncoder();
//	_this->p_current_adc[WHEEL_ID_A] = &adc_value.Left_Wheel_Current;		

	_this->config_dir[WHEEL_ID_B] = Hal_Wheel_Set_Dir_B;
	_this->config_enable[WHEEL_ID_B] = HwWheel_Config_Wheel_B;
	_this->p_tim_ccr[WHEEL_ID_B] = Hal_Wheel_GetCcr_B();
	_this->is_enable[WHEEL_ID_B] = DISABLE;
	_this->p_undir_encoder[WHEEL_ID_B] = Hal_Wheel_GetWheel_B_Addr_UndirEncoder();
	_this->p_dir_encoder[WHEEL_ID_B] = HwWheel_GetWheel_B_Addr_DirEncoder();
//	_this->p_current_adc[WHEEL_ID_B] = &adc_value.Left_Wheel_Current;		

	_this->config_dir[WHEEL_ID_C] = Hal_Wheel_Set_Dir_C;
	_this->config_enable[WHEEL_ID_C] = HwWheel_Config_Wheel_C;
	_this->p_tim_ccr[WHEEL_ID_C] = Hal_Wheel_GetCcr_C();
	_this->is_enable[WHEEL_ID_C] = DISABLE;
	_this->p_undir_encoder[WHEEL_ID_C] = Hal_Wheel_GetWheel_C_Addr_UndirEncoder();
	_this->p_dir_encoder[WHEEL_ID_C] = HwWheel_GetWheel_C_Addr_DirEncoder();
//	_this->p_current_adc[WHEEL_ID_C] = &adc_value.Left_Wheel_Current;			
}
void Wheel_Set_Speed(HwWheel_t *const _this)
{
	for(uint8_t i=0;i<3;i++)
	{
		_this->config_dir[i](_this->set_dir[i]);
		*_this->p_tim_ccr[i] = _this->set_spd[i];
	}
}
void Wheel_Controller_Update(HwWheel_t *const _this)
{	
	uint8_t i = 0;
	
//	if (!_this->manifest)return;

	if (_this->freeze_dir != WHEEL_IDLE)
	{
		if (Timer_Get_DiffNow(_this->freeze_time) > TIME_20MS)
		{
			LOG("%s(%d):release freeze,%d\n",__FUNCTION__,__LINE__,System_Get_Time());
			_this->freeze_dir = WHEEL_IDLE;
		}
	}


	for(i = 0;i < 2;i++)
	{
		if (_this->set_enable[i] != _this->is_enable[i])
		{	
			LOG("%s(%d):set enable:%d,spd:%d\n",__FUNCTION__,__LINE__,_this->set_enable[i],_this->set_spd[i]);
			_this->is_enable[i] = _this->set_enable[i];
			_this->config_enable[i](_this->is_enable[i]);	
			if (_this->is_enable[i])
			{
				*_this->p_tim_ccr[i] = 0;
				_this->set_spd[i] = 0;
			
				Wheel_Enable_Init(_this,(Wheel_Id_t)i);
				_this->unhealth[i] = false;
				*_this->p_tim_ccr[i] = 0;
				_this->baseline_cnt = -1; //disable
			}
			else
			{
				_this->config_dir[i](WHEEL_STOP);
			}
		}
		
		if (_this->is_enable[i])
		{
			if (_this->set_dir[i] != _this->dir[i])
			{
				LOG("%s(%d):i:%d,set dir:%d,set speed:%d\n",__FUNCTION__,__LINE__,i,_this->set_dir[i],_this->set_spd[i]);
				_this->dir[i] = _this->set_dir[i];
				_this->config_dir[i](_this->dir[i]);
				_this->change_time[i] = System_Get_Time();
				_this->pre_spd_delta[i] = 0;		
				*_this->p_tim_ccr[i] = 0;
			}
			
			if (_this->dir[i] != WHEEL_STOP)
			{
				if (System_Get_Time() >= (_this->change_time[i] + 250))
				{
					const int32_t dt_ms = (System_Get_Time()- _this->change_time[i])/10;
					_this->change_time[i] = System_Get_Time();
					Wheel_Calc_Spd(_this,(Wheel_Id_t)i,dt_ms);	
					Wheel_Pid_Control(_this,(Wheel_Id_t)i);	
			//		LOG("set dir:%d,%d,spd:(%d,%d),ttr(%d,%d)\n",_this->dir[0],_this->dir[1],_this->set_spd[0],_this->set_spd[1],*_this->p_tim_ccr[0],*_this->p_tim_ccr[1]);
				}
			}
		}
		else
		{
			if (_this->baseline_cnt >= 0)
			{
				Wheel_Update_BaseLine(_this);
			}
		}
	}
}



void Wheel_Calc_Spd(HwWheel_t *const _this,Wheel_Id_t id,uint16_t ms)
{
	int32_t diff;
	uint32_t dist_mm;
	diff = (*_this->p_undir_encoder[id] - _this->pre_undir_encoder[id]);
	if(diff < 0)diff = 0;
	if(diff > 1000)diff = 1000;
	
	dist_mm = diff*100*WHEEL_CIRCUMFERENCE_MM/WHEEL_REVOLUTION_COUNT;
	_this->pre_undir_encoder[id] = *_this->p_undir_encoder[id];
	_this->actual_spd[id] = (dist_mm*10/ms);
}

void Wheel_Pid_Control(HwWheel_t *const _this,Wheel_Id_t id)
{
	int32_t delta,p,d,out_put,delta_pre;
	
	delta = _this->set_spd[id] - _this->actual_spd[id];

	if (delta > 150)delta = 150;
	if (delta <-150)delta = -150;

	delta_pre = (delta - _this->pre_spd_delta[id]);

	if (delta_pre > 50)delta_pre = 50;
	if (delta_pre <-50)delta_pre = -50;
	
	p = delta*WHEEL_PID_P;
	d = delta_pre*WHEEL_PID_D;
	
	out_put = p + d;
	
	out_put+= *_this->p_tim_ccr[id];
		
	if (out_put > 99)out_put = 99;
	if (out_put < 0) out_put = 0;
	if (_this->set_spd[id] == 0)out_put = 0;

	_this->pre_spd_delta[id] = delta;
	*_this->p_tim_ccr[id] = out_put;
//	if (id == WHEEL_ID_RIGHT)
//	{
//		LOG_MB("set spd:%d,act:%d,delta:%d,pre:%d:p:%d,d:%d,output:%d\n",_this->set_spd[id],_this->actual_spd[id],delta,delta_pre,p,d,out_put);	
//	}
//	return 1;
}


void Wheel_Enable_Init(HwWheel_t *const _this,Wheel_Id_t id)
{
	_this->change_time[id] = System_Get_Time();
	_this->pre_spd_delta[id] = 0;
	_this->pre_undir_encoder[id] = 0;
	*_this->p_undir_encoder[id] = 0;
	
	*_this->p_dir_encoder[id] = 0;
	 _this->pre_dir_encoder[id] = 0;
}

Wheel_Dir_t Wheel_GetFreeze_Dir(void)
{
	return Robot_Get_Base()->wheel.freeze_dir;
}

Wheel_Dir_t Wheel_Get_RobotDir(void)
{
	HwWheel_t *const _this = &Robot_Get_Base()->wheel;
	
//	if ((_this->dir[WHEEL_ID_LEFT] == WHEEL_FORWARD) && (_this->dir[WHEEL_ID_RIGHT] == WHEEL_FORWARD))return WHEEL_FORWARD;
//	if ((_this->dir[WHEEL_ID_LEFT] == WHEEL_BACKWARD) && (_this->dir[WHEEL_ID_RIGHT] == WHEEL_BACKWARD))return WHEEL_BACKWARD;
//	if ((_this->dir[WHEEL_ID_LEFT] == WHEEL_FORWARD) && (_this->dir[WHEEL_ID_RIGHT] == WHEEL_BACKWARD))return WHEEL_TURN_RIGHT;
//	if ((_this->dir[WHEEL_ID_LEFT] == WHEEL_BACKWARD) && (_this->dir[WHEEL_ID_RIGHT] == WHEEL_FORWARD))return WHEEL_TURN_LEFT;
	
	return WHEEL_STOP;
}


void Wheel_Freeze(Wheel_Dir_t dir)
{
	LOG("%s(%d),time:%d\n",__FUNCTION__,__LINE__,System_Get_Time());
	HwWheel_t *const _this = &Robot_Get_Base()->wheel;
	_this->freeze_dir = dir;
	_this->freeze_time = System_Get_Time();
	
	*_this->p_tim_ccr[0] = 0;
	*_this->p_tim_ccr[1] = 0;
	_this->set_dir[0] = WHEEL_STOP;
	_this->set_dir[1] = WHEEL_STOP;
}


void Wheel_SetDirSpeeds(HwWheel_t *const _this,Wheel_Dir_t dir,uint16_t spd)
{
	_this->set_dir[0] = ((dir == WHEEL_FORWARD)||(dir == WHEEL_TURN_RIGHT))? WHEEL_FORWARD : WHEEL_BACKWARD;
	_this->set_dir[1] = ((dir == WHEEL_FORWARD)||(dir == WHEEL_TURN_LEFT))?  WHEEL_FORWARD : WHEEL_BACKWARD;

	_this->set_spd[0] = spd;
	_this->set_spd[1] = spd;
	
	if(_this->set_spd[0] == 0)*_this->p_tim_ccr[0] = 0;
	if(_this->set_spd[1] == 0)*_this->p_tim_ccr[1] = 0;	
}

void Wheel_SetSpeeds(HwWheel_t *const _this,uint16_t spd_l,uint16_t spd_r)
{	
	_this->set_spd[0] = spd_l;
	_this->set_spd[1] = spd_r;
	
	if(_this->set_spd[0] == 0)*_this->p_tim_ccr[0] = 0;
	if(_this->set_spd[1] == 0)*_this->p_tim_ccr[1] = 0;	
}


void Wheel_SetDir(HwWheel_t *const _this,Wheel_Dir_t dir)
{
	_this->set_dir[0] = ((dir == WHEEL_FORWARD)||(dir == WHEEL_TURN_RIGHT))? WHEEL_FORWARD : WHEEL_BACKWARD;
	_this->set_dir[1] = ((dir == WHEEL_FORWARD)||(dir == WHEEL_TURN_LEFT))?  WHEEL_FORWARD : WHEEL_BACKWARD;

}


void Wheel_Stop(HwWheel_t *const _this)
{
	uint8_t id = 0;
	
	for (id =0;id < 2;id++)
	{
		_this->set_spd[id] = 0;
		_this->set_enable[id] = DISABLE;
		_this->is_enable[id] = DISABLE;
		*_this->p_tim_ccr[id] = 0;
		_this->set_dir[id] = WHEEL_STOP;
		_this->dir[id] = WHEEL_STOP;
		_this->current[id] = 0;
		_this->config_dir[id](WHEEL_STOP);
	}
}


void Wheel_Reset_BaseLine(HwWheel_t *const _this)
{
	_this->baseline_cnt = 0;
	_this->baseline_sum[0] = 0;
	_this->baseline_sum[1] = 1;
}


void Wheel_Update_BaseLine(HwWheel_t *const _this)
{
	static uint32_t pre_time;

	if(_this->baseline_cnt >= 0)
	{
		if(_this->is_enable[0] || _this->is_enable[1]) //robot stop update if start running
		{
			LOG("%s(%d):robot stop update,because start running,use last data\n",__FUNCTION__,__LINE__);
			_this->baseline_cnt = -1;
			return;
		}
	
		if (Timer_Is_TimeReach(&pre_time,TIME_20MS))
		{
			_this->baseline_sum[0] += *_this->p_current_adc[0];
			_this->baseline_sum[1] += *_this->p_current_adc[1];	
		
			if(++_this->baseline_cnt >= 10)
			{
				_this->baseline[0] = _this->baseline_sum[0]/_this->baseline_cnt;
				_this->baseline[1] = _this->baseline_sum[1]/_this->baseline_cnt;
				_this->baseline_cnt = -1;
				LOG("%s(%d):baseline:%d,%d\n",__FUNCTION__,__LINE__,_this->baseline[0],_this->baseline[1]);
			}
		}
	}
} 



void Wheel_Health_Update(HwWheel_t *const _this)
{
	uint8_t i = 0;

	if (Timer_Is_TimeReach(&_this->update_health_time[i],HEALTH_WHEEL_TIME_CYCLE)) //50ms
	{
		uint8_t is_warning;
		
		for (i = 0;i < 2; i++)
		{
			if (!_this->is_enable[i] || (_this->set_spd[i] == 0))
			{
				continue;
			}
		
			if (*_this->p_current_adc[i] > _this->baseline[i])
			{
				_this->current_sum[i] += (*(_this->p_current_adc[i]) - _this->baseline[i])*ADC_TO_VOLTAGE_WHEEL;
				_this->current_cnt[i]++;
			}
			
			if(_this->current_cnt[i] >= 10) //500MS
			{
				_this->current[i] = _this->current_sum[i]/_this->current_cnt[i];
				_this->current_sum[i] = 0;
				_this->current_cnt[i] = 0;
						
				is_warning = false;		
				if ((_this->current[i] < HEALTH_WHEEL_CURRENT_LIMIT_L) && (*_this->p_tim_ccr[i] > 50))
				{
					_this->lower_current_cnt[i]++;
					is_warning = true;				
				}
				if(_this->current[i] > HEALTH_WHEEL_CURRENT_LIMIT_H)
				{
					_this->over_current_cnt[i]++;
					is_warning = true;
				}
				
				if (_this->set_spd[i] > 100 && _this->actual_spd[i] < _this->set_spd[i]*0.7)
				{
					_this->speed_lose_cnt[i]++;	
					is_warning = true;	
				}

				if(_this->lower_current_cnt[i] || _this->over_current_cnt[i] || _this->speed_lose_cnt[i])
				{
			//		LOG_W("Wheel unhealth:i:%d,cnt[%d,%d],p_adc:%d,base:%d,current:%d,spd:%d,%d\n",i,_this->lower_current_cnt[i],_this->over_current_cnt[i],*_this->p_current_adc[i],_this->baseline[i],_this->current[i],_this->set_spd[i],_this->actual_spd[i]);	
					
					if ((_this->speed_lose_cnt[i] > HEALTH_WHEEL_ERROR_CNT) || (_this->lower_current_cnt[i] > HEALTH_WHEEL_ERROR_CNT)||(_this->over_current_cnt[i]  > HEALTH_WHEEL_ERROR_CNT))//2s
					{
						LOG("%s(%d):current error:current:%d,range[%d,%d]\n",__FUNCTION__,__LINE__,_this->current[i],HEALTH_WHEEL_CURRENT_LIMIT_L,HEALTH_WHEEL_CURRENT_LIMIT_H);
						_this->unhealth[i] = true;
						if (_this->on_health_listener) 
						{
							_this->on_health_listener();
						}	
						_this->lower_current_cnt[i] = 0;//HEALTH_WHEEL_ERROR_CNT/2;
						_this->over_current_cnt[i] = 0;//HEALTH_WHEEL_ERROR_CNT/2;	
						_this->speed_lose_cnt[i] = 0;
					}
					else if(!is_warning)
					{
						if(_this->lower_current_cnt[i] > 0)_this->lower_current_cnt[i]--;
						if(_this->over_current_cnt[i] > 0)_this->over_current_cnt[i]--;
						if(_this->speed_lose_cnt[i] > 0)_this->speed_lose_cnt[i]--;
					}
				}	
			}
		}
	}
}

uint32_t Wheel_GetRobot_UndirEncoder(void)
{
	HwWheel_t *const _this = &Robot_Get_Base()->wheel;
	return (*_this->p_undir_encoder[0] + *_this->p_undir_encoder[1]);
}


void Wheel_Set_OnHealth_Listener(HwWheel_t *const _this,HwWheelOnHealthListener onhealth_fun)
{
	_this->on_health_listener = onhealth_fun;
}

void Wheel_OnHealth_Listener(void)
{
//	Linux_Report_Event(LINUX_UPLINK_EVENT_HEALTH);
}

