#ifndef __driving_wheel_h
#define __driving_wheel_h	 
#ifdef __cplusplus
 extern "C" {
#endif
#include "global_param.h"

#define	ADC_TO_VOLTAGE_WHEEL			((float)330*10/4096)
	 
#define WHEEL_CIRCUMFERENCE_MM             2198  //70mm*3.14
#define WHEEL_REVOLUTION_COUNT             11232

#define WHEEL_PID_P                        0.09
#define WHEEL_PID_D                        0.08 //0.13

#define HEALTH_ERROR_LIMIT                 80      //80*250 == 2s  
#define HEALTH_WHEEL_TIME_CYCLE            500
#define HEALTH_WHEEL_CURRENT_LIMIT_H       500
#define HEALTH_WHEEL_CURRENT_LIMIT_L       10
#define HEALTH_WHEEL_ERROR_CNT             3  // 10   			  

typedef enum
{
	WHEEL_IDLE,
	WHEEL_FORWARD,
	WHEEL_BACKWARD,
	WHEEL_STOP,
	WHEEL_TURN_LEFT,
	WHEEL_TURN_RIGHT,
} Wheel_Dir_t;
typedef void(*HwWheelSetDir)(Wheel_Dir_t state);
typedef void(*HwWheelConfig)(uint8_t state);
typedef void(*HwWheelOnHealthListener)(void);


typedef enum
{
	WHEEL_ID_A,
	WHEEL_ID_B,
	WHEEL_ID_C,
} Wheel_Id_t;



typedef struct
{
	bool manifest;

	HwWheelSetDir config_dir[3];
	HwWheelConfig config_enable[3];
	volatile uint16_t *p_tim_ccr[3];
	uint8_t is_enable[3];
	uint8_t set_enable[3];
	Wheel_Dir_t set_dir[3];
	Wheel_Dir_t dir[3];

	uint8_t current_cnt[3];
	uint32_t change_time[3];
	uint32_t current_sum[3];
	int16_t set_spd[3];
	uint16_t actual_spd[3];
	volatile int32_t *p_undir_encoder[3];
	volatile int32_t *p_dir_encoder[3];
	int32_t pre_dir_encoder[3];
	int32_t pre_undir_encoder[3];
	int16_t pre_spd_delta[3];
	
	Wheel_Dir_t freeze_dir;
	uint32_t freeze_time;
	
	uint32_t baseline_sum[3];
	uint16_t baseline[3];
	int8_t baseline_cnt;
	
	volatile uint16_t current[3];
	uint32_t update_health_time[3];
	uint16_t lower_current_cnt[3];	
	uint16_t over_current_cnt[3];
	uint8_t speed_lose_cnt[3];
	uint16_t baseline_count[3];
	bool is_baseline_ready[3];
	

	volatile uint16_t *p_current_adc[3];
	bool unhealth[3];
	HwWheelOnHealthListener on_health_listener;
} HwWheel_t;


void Wheel_Construct(HwWheel_t *const _this);
void Wheel_Controller_Update(HwWheel_t *const _this);
void Wheel_EncoderL_Isr(void);
void Wheel_EncoderR_Isr(void);
Wheel_Dir_t Wheel_Get_RobotDir(void);
void Wheel_Calc_Spd(HwWheel_t *const _this,Wheel_Id_t id,uint16_t ms);
void Wheel_Pid_Control(HwWheel_t *const _this,Wheel_Id_t id);
void Wheel_Enable_Init(HwWheel_t *const _this,Wheel_Id_t id);
void Wheel_HealthListener(HwWheel_t *const _this);
void Wheel_Freeze(Wheel_Dir_t dir);
void Wheel_SetDir(HwWheel_t *const _this,Wheel_Dir_t dir);
void Wheel_Stop(HwWheel_t *const _this);
void Wheel_SetSpeeds(HwWheel_t *const _this,uint16_t spd_l,uint16_t spd_r);
void Wheel_SetDirSpeeds(HwWheel_t *const _this,Wheel_Dir_t dir,uint16_t spd);
Wheel_Dir_t Wheel_GetFreeze_Dir(void);
void Wheel_Health_Update(HwWheel_t *const _this);
void Wheel_OnHealth_Listener(void);
void Wheel_Set_OnHealth_Listener(HwWheel_t *const _this,HwWheelOnHealthListener onhealth_fun);
void Wheel_Update_BaseLine(HwWheel_t *const _this);
void Wheel_Reset_BaseLine(HwWheel_t *const _this);

uint32_t Wheel_GetRobot_UndirEncoder(void);

void Wheel_Set_Speed(HwWheel_t *const _this);
#ifdef __cplusplus
}
#endif
#endif
