#include "led.h"
#include "delay.h"

/***************************************************************/
LedCtrl::LedCtrl():LedIIC()
{	
    address = DEFAULT_WIRTE_ADDRESS;
	temp_add = address;	
	init_state = false;	
	ctrl_gpiox = GPIOB;
	ctrl_pin = GPIO_Pin_9;
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Pin = ctrl_pin ;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;  	/* 开漏输出 */
	GPIO_Init(ctrl_gpiox, &GPIO_InitStructure);	
}

LedCtrl &LedCtrl::GetInstance()
{
	static LedCtrl  m_instance;
	return m_instance;
}

static uint8_t __IS31FL3236_Select_Color(uint8_t color_selector, uint8_t r, uint8_t g, uint8_t b);

/*
 * Initialises the given handle.
 */
void LedCtrl::IS31FL3236_Init(IS31FL3236_HandleTypeDef* handle)
{
	IS31FL3236_Reset(handle);
	IS31FL3236_SetSoftwareShutdown(handle, IS31FL3236_SOFTWARE_SHUTDOWN_DISABLED);
	IS31FL3236_SetChipEnable(handle, IS31FL3236_CHIP_ENABLED);
}

/*
 * Changes the chip enable state of the specified chip.
 */
void LedCtrl::IS31FL3236_SetChipEnable(IS31FL3236_HandleTypeDef* handle, uint8_t enable_state)
{
	
//	HAL_GPIO_WritePin(handle->Init.Chip_Enable_Signal_Port, handle->Init.Chip_Enable_Signal_Pin, enable_state);
	StartLedIc();
}

/*
 * Writes a given value to a given register.
 */
void LedCtrl::IS31FL3236_WriteRegister(IS31FL3236_HandleTypeDef* handle, uint8_t register_address, uint8_t value)
{
//	uint8_t buf[2];
//	buf[0] = register_address;
//	buf[1] = value;
//	HAL_I2C_Master_Transmit(handle->Init.I2C_Bus, handle->Init.I2C_Device_Address, buf, 2, handle->Init.I2C_Transmit_Timeout_Milliseconds);
	i2c_Start();
	i2c_SendByte(DEFAULT_WIRTE_ADDRESS);
	i2c_WaitAck();
	i2c_SendByte(register_address);
	i2c_WaitAck();
	i2c_SendByte(value);
	i2c_WaitAck();
	i2c_Stop();
}

/*
 * Resets the specified chip. (Through I2C)
 */
void LedCtrl::IS31FL3236_Reset(IS31FL3236_HandleTypeDef* handle)
{
	IS31FL3236_WriteRegister(handle, IS31FL3236_REGISTER_RESET, 0x00);
}

/*
 * Latches the written PWM and led control values in the chip to the led control circuit.
 */
void LedCtrl::IS31FL3236_Update(IS31FL3236_HandleTypeDef* handle)
{
	IS31FL3236_WriteRegister(handle, IS31FL3236_REGISTER_UPDATE, 0x00);
}

/*
 * Sets the LED control for the specified chip / channel.
 */
void LedCtrl::IS31FL3236_WriteLEDControl(IS31FL3236_HandleTypeDef* handle, uint8_t channel, uint8_t led_current_setting, uint8_t led_state)
{
	if (channel < IS31FL3236_MAX_CHANNELS)
		IS31FL3236_WriteRegister(handle, IS31FL3236_REGISTER_LED_CTRL + channel, (led_current_setting | led_state));
}

/*
 * Writes an led control word to all channels of a specified chip.
 */
void LedCtrl::IS31FL3236_WriteGlobalLEDControl(IS31FL3236_HandleTypeDef* handle, uint8_t led_current_setting, uint8_t led_state)
{
	for (uint8_t i = 0; i < IS31FL3236_MAX_CHANNELS; i++)
	{
		IS31FL3236_WriteLEDControl(handle, i, led_current_setting, led_state);
	}
}

/*
 * Writes the given PWM value (0 - 255) to the specified chip / channel.
 */
void LedCtrl::IS31FL3236_WritePWM(IS31FL3236_HandleTypeDef* handle, uint8_t channel, uint8_t pwm_value)
{
	if (channel < IS31FL3236_MAX_CHANNELS)
		IS31FL3236_WriteRegister(handle, IS31FL3236_REGISTER_PWM + channel, pwm_value);
}

/*
 * Writes a given color (r, g, b) to a specified rgb channel of the specified chip. The periodic order the individual leds
 * are connected can be set in the Init attribute of the handle struct. (RGB_Mode_Color_x)
 */
void LedCtrl::IS31FL3236_WriteRGBPWM(IS31FL3236_HandleTypeDef* handle, uint8_t rgb_channel, uint8_t red, uint8_t green, uint8_t blue)
{
	if (rgb_channel < IS31FL3236_MAX_RGB_CHANNELS)
	{
		IS31FL3236_WritePWM(handle, rgb_channel * 3, __IS31FL3236_Select_Color(handle->Init.RGB_Mode_Color_1, red, green, blue));
		IS31FL3236_WritePWM(handle, (rgb_channel * 3) + 1, __IS31FL3236_Select_Color(handle->Init.RGB_Mode_Color_2, red, green, blue));
		IS31FL3236_WritePWM(handle, (rgb_channel * 3) + 2, __IS31FL3236_Select_Color(handle->Init.RGB_Mode_Color_3, red, green, blue));
	}
}

/*
 * Sets the software shutdown mode of the specified chip.
 */
void LedCtrl::IS31FL3236_SetSoftwareShutdown(IS31FL3236_HandleTypeDef* handle, uint8_t software_shutdown_mode)
{
	IS31FL3236_WriteRegister(handle, IS31FL3236_REGISTER_SHUTDOWN, software_shutdown_mode);
}




/*
 * Selects red, green or blue value by the given color selector.
 */
static uint8_t __IS31FL3236_Select_Color(uint8_t color_selector, uint8_t r, uint8_t g, uint8_t b)
{
	switch (color_selector)
	{
	case IS31FL3236_RGB_CONFIG_RED:
		return r;
	case IS31FL3236_RGB_CONFIG_GREEN:
		return g;
	case IS31FL3236_RGB_CONFIG_BLUE:
		return b;
	default:
		return 0;
	}
}
void LedCtrl::IS31FL3236_WriteFreqCTRL(IS31FL3236_HandleTypeDef* handle)
{
	IS31FL3236_WriteRegister(handle, IS31FL3236_REGISTER_FREQ_CTRL, 0x00);
}
void LedCtrl::IS31FL3236_WriteGlobalLEDControlEn(IS31FL3236_HandleTypeDef* handle)
{
	IS31FL3236_WriteRegister(handle, IS31FL3236_REGISTER_GLOBAL_CTRL, 0x00);
}
void LedCtrl::IS31FL3236_WriteAllPWM(IS31FL3236_HandleTypeDef* handle,  uint8_t pwm_value)
{
	for (uint8_t i = 0; i < IS31FL3236_MAX_CHANNELS; i++)
	{
		IS31FL3236_WritePWM(handle, i, pwm_value);
	}
}
void LedCtrl::IS31FL3236_WriteAllRGBPWM(IS31FL3236_HandleTypeDef* handle, uint8_t red, uint8_t green, uint8_t blue)
{
	for (uint8_t i = 0; i < IS31FL3236_MAX_RGB_CHANNELS; i++)
	{
		IS31FL3236_WriteRGBPWM(handle, i, red,green,blue);
	}
}
void LedCtrl::Init(void)
{
//	StartLedIc();
	LOG("LedCtrl Init and power on\n");
	Timer_Delay_Ms(1);
	IS31FL3236_Init(ptr_h_led_drvr_foo);
	
	IS31FL3236_HandleTypeDef h_led_drvr_foo;
//	h_led_drvr_foo.I2C_Bus = hi2c1;
	h_led_drvr_foo.Init.I2C_Device_Address = IS31FL3236_GET_I2C_ADDR(IS31FL3236_I2C_AD_TO_GND);
	h_led_drvr_foo.Init.I2C_Transmit_Timeout_Milliseconds = 10 ; // 10ms 是默认值。
	h_led_drvr_foo.Init.Chip_Enable_Signal_Port = GPIOB;
	h_led_drvr_foo.Init.Chip_Enable_Signal_Pin = GPIO_Pin_9;

	//可选 [
	h_led_drvr_foo.Init.RGB_Mode_Color_1 = IS31FL3236_RGB_CONFIG_RED;
	h_led_drvr_foo.Init.RGB_Mode_Color_2 = IS31FL3236_RGB_CONFIG_GREEN;
	h_led_drvr_foo.Init.RGB_Mode_Color_3 = IS31FL3236_RGB_CONFIG_BLUE;
	// ]

	//将指向句柄的指针存储在（全局）变量中，以便将来在其他函数中访问。
	ptr_h_led_drvr_foo = &h_led_drvr_foo;

	//初始化句柄。
	IS31FL3236_Init (ptr_h_led_drvr_foo);
	// Set the basic global LED current mode and state settings. (Turn on all channels to the max. current)
	IS31FL3236_WriteGlobalLEDControl(ptr_h_led_drvr_foo, IS31FL3236_LED_CURRENT_MAX, IS31FL3236_LED_STATE_ON);
	// Send an update command to the chip.
	IS31FL3236_Update(ptr_h_led_drvr_foo);	
	
	// Set the PWM value of channel (ID) 8 to 200.
	IS31FL3236_WritePWM(ptr_h_led_drvr_foo, 0, 40);
	// Send the chip an update command.
	IS31FL3236_Update(ptr_h_led_drvr_foo);	
	


//	IS31FL3236_WriteFreqCTRL(ptr_h_led_drvr_foo);
//	IS31FL3236_WriteGlobalLEDControlEn(ptr_h_led_drvr_foo);
	
//	IS31FL3236_WriteRGBPWM(ptr_h_led_drvr_foo, 0, 0xff, 0x00, 0x00);
//	IS31FL3236_WriteRGBPWM(ptr_h_led_drvr_foo, 1, 0xff, 0x00, 0x00);
	IS31FL3236_WriteAllRGBPWM(ptr_h_led_drvr_foo, 0x00, 0xff, 0x00);
// Send the chip an update command.
	IS31FL3236_Update(ptr_h_led_drvr_foo);	
}

void LedCtrl::Process(void)
{
//	for(uint8_t i=0;i<0x1f;i++)
//	{
//		IS31FL3236_WriteAllRGBPWM(ptr_h_led_drvr_foo, i, i, i);
//		IS31FL3236_Update(ptr_h_led_drvr_foo);	
//	}
//	for(uint8_t i=0x1f;i>0;i--)
//	{
//		IS31FL3236_WriteAllRGBPWM(ptr_h_led_drvr_foo, i, i, i);
//		IS31FL3236_Update(ptr_h_led_drvr_foo);	
//	}	
//	return;
	
	
	static uint8_t pwm_cnt =0,sta =0;

	switch(sta)
	{
		case 0:
		if(pwm_cnt<0xff)
		{
			IS31FL3236_WriteAllRGBPWM(ptr_h_led_drvr_foo, pwm_cnt, pwm_cnt, pwm_cnt);
			IS31FL3236_Update(ptr_h_led_drvr_foo);	
			pwm_cnt++;
		}
		else
			sta =1;	
		
		break;
		case 1:
		if(pwm_cnt>0)
		{
			IS31FL3236_WriteAllRGBPWM(ptr_h_led_drvr_foo, pwm_cnt, pwm_cnt, pwm_cnt);
			IS31FL3236_Update(ptr_h_led_drvr_foo);	
			pwm_cnt--;
		}	
		else
			sta =0;
		break;
	}

}


