#include "robot_ctrl.h"
#include "delay.h"
#include "usart.h"
#include "led.h"
#include "driving_wheel.h"
#include "hal_adc.h"
#include "hal_imu.h"
#include "hal_oled.h"
#include "hal_rtc.h"
#include "test_main.h"
#include "voice_interaction.h"
#include "hal_air_quality_sensor.h"
#include "hal_infrared.h"
#include "external_flash.h"
#include "hal_ultrasonic.h"
#include "LD3320_Process.h"

extern const ST_ASR_DATA pAsrItem[];
Robot_t Robot;
void Robot_Config_Manifest(Robot_t *const _this,uint32_t manifest)
{
	_this->wheel.manifest     = ( manifest & MAINFEST_MOTOR_WHEEL     )? true  : false;    
//	_this->mainbrush.manifest = ( manifest & MAINFEST_MOTOR_MAINBRUSH )? true : false;  
//	_this->sidebrush.manifest = ( manifest & MAINFEST_MOTOR_BRUSH     )? true : false;     
//	_this->vacuum.manifest    = ( manifest & MAINFEST_MOTOR_VACUUM    )? true : false;   
#ifdef HAS_WIND
	_this->pump.manifest = ( manifest & MAINFEST_MOTOR_WIND    )? true : false;   
#endif
//	_this->led.manifest       = ( manifest & MAINFEST_EVENT_LED       )? true : false;   
//	_this->vacuum.manifest    = ( manifest & MAINFEST_MOTOR_VACUUM    )? true : false;   
//	_this->button.manifest    = ( manifest & MAINFEST_EVENT_BUTTON    )? true : false;        
//	_this->bumper.manifest    = ( manifest & MAINFEST_EVENT_BUMPER    )? true : false; 
//	_this->scanwall.manifest    = ( manifest & MAINFEST_EVENT_SCAN_WALL    )? true : false; 
//	_this->cliff.manifest    = ( manifest & MAINFEST_EVENT_CLIFF    )? true : false; 
//	_this->wheelsuspend.manifest =( manifest & MAINFEST_EVENT_SUSPEND  )? true : false; 

#ifdef HAS_PUMP
	_this->pump.manifest = ( manifest & MAINFEST_PUMP )? true : false;   
#endif

}
void System_Init(void)
{
	Uart_Init();
	RTC_Init();	
	Hal_Adc_Init();
	Hal_Oled_Init();
	hal_imu_init(); 
	LedCtrl::GetInstance().Init(); 		
	Wheel_Construct(&Robot_Get_Base()->wheel);
	IAR_Init();
	External_Flash_Init();
	Infrared_Init();
	Ultrasouic_Cap_Init();		
}

/************************************/
void System_Process(void)
{
	imu_process();
	Hal_Oled_Test_Disp();
	AdcRead();
	Voice_Iteraction_Process();
	IAR_Process();
	External_Flash_Process();
	Ultrasouic_Process();
//	Wheel_Controller_Update(&Robot_Get_Base()->wheel);
}
/*******************************/
void Is_Play_Delay(void)
{
	while(bMp3Play) {
		LOG("*********is playing*********\r\n");
		Timer_Delay_Ms(10);
	}
}

void Task_Respose(void)
{
	bMp3Play = 1;
	LD_Init_MP3();
	LD_Adjust_Volume(10);	
	PlaySoundFromFLASH(5);		
}
void Task_Air_Quality(void)
{
	Is_Play_Delay();
	bMp3Play = 1;
	LD_Init_MP3();
	LD_Adjust_Volume(10);
	PlaySoundFromFLASH(9);		
	Is_Play_Delay();

	if(Get_IAR_PPM()<0.3)
		PlaySoundFromFLASH(10);	
	else
		PlaySoundFromFLASH(3);			
	Timer_Delay_Ms(100);
	Force_Clear_Play();
	
	PlaySoundFromFLASH(15);		
	Is_Play_Delay();
	int ppm=  Get_IAR_PPM()/10;
	Play_Number_Packet((SpeakerType)1,10);
	Timer_Delay_Ms(100);
	Force_Clear_Play();
	PlaySoundFromFLASH(6);	/*point*/
	Timer_Delay_Ms(100);
	Force_Clear_Play();		
	Play_Number_Packet((SpeakerType)3,10);
	Timer_Delay_Ms(100);
	Force_Clear_Play();
	
	Is_Play_Delay();	
	PlaySoundFromFLASH(19);		
	Timer_Delay_Ms(500);
	Force_Clear_Play();		
	int temp = (int)Get_IAR_Temperature();
	int humi = (int)Get_IAR_Humidity();
	int hundredth =0,tenth=0,oneth =0;
	hundredth =temp/100,tenth=temp/10,oneth =temp%10;
	Is_Play_Delay();
	if(tenth > 0) 
	{
		Play_Number_Packet((SpeakerType)tenth,10);
		Timer_Delay_Ms(100);
		Force_Clear_Play();
		Play_Number_Packet((SpeakerType)10,10);
		Timer_Delay_Ms(100);
		Force_Clear_Play();		
	}
	Is_Play_Delay();

	if(oneth > 0) 
		Play_Number_Packet((SpeakerType)oneth,10);	
	Is_Play_Delay();
	PlaySoundFromFLASH(16);		
	Timer_Delay_Ms(500);
	Force_Clear_Play();	

	Is_Play_Delay();	
	PlaySoundFromFLASH(18);	/*ʪ*/	
	Timer_Delay_Ms(500);
	Force_Clear_Play();	
	
	PlaySoundFromFLASH(17);		
	Timer_Delay_Ms(500);
	Force_Clear_Play();	
	hundredth =humi/100,tenth=humi/10,oneth =humi%10;

	Is_Play_Delay();
	if(tenth > 0) 
	{
		Play_Number_Packet((SpeakerType)tenth,10);
		Timer_Delay_Ms(100);
		Force_Clear_Play();	
		Play_Number_Packet((SpeakerType)10,10);
		Timer_Delay_Ms(100);
		Force_Clear_Play();		
	}
	Is_Play_Delay();
	if(oneth > 0) 
		Play_Number_Packet((SpeakerType)oneth,10);		
	Is_Play_Delay();
}
void Task_Test(void)
{
	Timer_Delay_Ms(50);
	Play_Number_Packet((SpeakerType)1,10);
	Timer_Delay_Ms(100);
	Play_Number_Packet((SpeakerType)2,10);
	Timer_Delay_Ms(100);
	Play_Number_Packet((SpeakerType)3,10);
	Timer_Delay_Ms(100);
	Play_Number_Packet((SpeakerType)4,10);
	Timer_Delay_Ms(100);
	Play_Number_Packet((SpeakerType)5,10);
	Timer_Delay_Ms(100);
	Play_Number_Packet((SpeakerType)6,10);
	Timer_Delay_Ms(100);
	Play_Number_Packet((SpeakerType)7,10);
	Timer_Delay_Ms(100);
	Play_Number_Packet((SpeakerType)8,10);
	Timer_Delay_Ms(100);
	Play_Number_Packet((SpeakerType)9,10);
	Timer_Delay_Ms(100);
	Play_Number_Packet((SpeakerType)10,10);
	Timer_Delay_Ms(100);
	Play_Number_Packet((SpeakerType)11,10);
	Timer_Delay_Ms(100);	
}
void Task_Time(void)
{
	bMp3Play = 1;
	LD_Init_MP3();
	LD_Adjust_Volume(10);	
	PlaySoundFromFLASH(4);		
	Is_Play_Delay();

	RTC_Get();
	
	int tenth_hour=0,oneth_hour =0,tenth_minu=0,oneth_minu =0;
	tenth_hour=calendar.hour/10
	,oneth_hour =calendar.hour%10;
	tenth_minu=calendar.min/10
	,oneth_minu =calendar.min%10;
	if(tenth_hour > 0) 
	{
		if(tenth_hour==1)
		{
			Play_Number_Packet((SpeakerType)10,10);
		}
		if(tenth_hour==2)
		{
			Play_Number_Packet((SpeakerType)11,10);
		}
		Is_Play_Delay();
	}
	if(oneth_hour > 0) 
		Play_Number_Packet((SpeakerType)oneth_hour,10);
	Is_Play_Delay();
	PlaySoundFromFLASH(6);	/*point*/
	Timer_Delay_Ms(100);
	Force_Clear_Play();	
//	PlaySoundFromFLASH(7);	
//	Timer_Delay_Ms(100);
//	Force_Clear_Play();
	
	if(tenth_minu > 0)
	{
		Play_Number_Packet((SpeakerType)tenth_minu,10);
	Is_Play_Delay();
		Play_Number_Packet((SpeakerType)10,10);
	}
	
	Is_Play_Delay();
	if(oneth_minu > 0) 
		Play_Number_Packet((SpeakerType)oneth_minu,10);
	Is_Play_Delay();
	PlaySoundFromFLASH(8);	
	Timer_Delay_Ms(100);
	Force_Clear_Play();

	
//	Is_Play_Delay();
//	Play_Packet(NINE,10);	
//	Is_Play_Delay();
//	Play_Packet(POINT,10);	
//	Is_Play_Delay();
//	Play_Packet(COMPLETE,10);	
//	Is_Play_Delay();
}

void Task_Forward(void)
{
	bMp3Play = 1;
	LD_Init_MP3();
	LD_Adjust_Volume(10);	
	PlaySoundFromFLASH(2);	
	Is_Play_Delay();
	PlaySoundFromFLASH(11);		
	Timer_Delay_Ms(700);
	Force_Clear_Play();
	Robot_Get_Base()->wheel.set_dir[WHEEL_ID_A] = WHEEL_BACKWARD;
	Robot_Get_Base()->wheel.set_dir[WHEEL_ID_B] = WHEEL_FORWARD;
	Robot_Get_Base()->wheel.set_dir[WHEEL_ID_C] = WHEEL_STOP;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_A] = 800;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_B] = 800;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_C] = 800;
	Wheel_Set_Speed(&Robot_Get_Base()->wheel);
	Timer_Delay_Ms(6000);
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_A] = 0;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_B] = 0;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_C] = 0;
	Wheel_Set_Speed(&Robot_Get_Base()->wheel);
}
void Task_Backward(void)
{
	bMp3Play = 1;
	LD_Init_MP3();
	LD_Adjust_Volume(10);	
	PlaySoundFromFLASH(2);	
	Is_Play_Delay();
	PlaySoundFromFLASH(12);		
	Timer_Delay_Ms(700);
	Force_Clear_Play();
	Robot_Get_Base()->wheel.set_dir[WHEEL_ID_A] = WHEEL_FORWARD;
	Robot_Get_Base()->wheel.set_dir[WHEEL_ID_B] = WHEEL_BACKWARD;
	Robot_Get_Base()->wheel.set_dir[WHEEL_ID_C] = WHEEL_STOP;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_A] = 800;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_B] = 800;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_C] = 800;	
	Wheel_Set_Speed(&Robot_Get_Base()->wheel);
	Timer_Delay_Ms(6000);
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_A] = 0;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_B] = 0;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_C] = 0;
	Wheel_Set_Speed(&Robot_Get_Base()->wheel);	
}
void Task_Turn_Left(void)
{
	bMp3Play = 1;
	LD_Init_MP3();
	LD_Adjust_Volume(10);	
	PlaySoundFromFLASH(2);	
	Is_Play_Delay();
	PlaySoundFromFLASH(13);		
	Timer_Delay_Ms(700);
	Force_Clear_Play();
	Robot_Get_Base()->wheel.set_dir[WHEEL_ID_A] = WHEEL_BACKWARD;
	Robot_Get_Base()->wheel.set_dir[WHEEL_ID_B] = WHEEL_BACKWARD;
	Robot_Get_Base()->wheel.set_dir[WHEEL_ID_C] = WHEEL_BACKWARD;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_A] = 800;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_B] = 800;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_C] = 800;		
	Wheel_Set_Speed(&Robot_Get_Base()->wheel);
	Timer_Delay_Ms(6000);
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_A] = 0;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_B] = 0;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_C] = 0;
	Wheel_Set_Speed(&Robot_Get_Base()->wheel);	
}
void Task_Turn_Right(void)
{
	bMp3Play = 1;
	LD_Init_MP3();
	LD_Adjust_Volume(10);	
	PlaySoundFromFLASH(2);	
	Is_Play_Delay();
	PlaySoundFromFLASH(14);		
	Timer_Delay_Ms(700);
	Force_Clear_Play();
	Robot_Get_Base()->wheel.set_dir[WHEEL_ID_A] = WHEEL_FORWARD;
	Robot_Get_Base()->wheel.set_dir[WHEEL_ID_B] = WHEEL_FORWARD;
	Robot_Get_Base()->wheel.set_dir[WHEEL_ID_C] = WHEEL_FORWARD;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_A] = 800;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_B] = 800;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_C] = 800;			
	Wheel_Set_Speed(&Robot_Get_Base()->wheel);
	Timer_Delay_Ms(6000);
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_A] = 0;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_B] = 0;
	Robot_Get_Base()->wheel.set_spd[WHEEL_ID_C] = 0;
	Wheel_Set_Speed(&Robot_Get_Base()->wheel);	
}
void Task_Music(void)
{
	bMp3Play = 1;
	LD_Init_MP3();
	LD_Adjust_Volume(10);		
	PlaySoundFromFLASH(0);			
}
void Task_Next_Music(void)
{
	bMp3Play = 1;
	LD_Init_MP3();
	LD_Adjust_Volume(10);	
	if(nMp3Size==pAsrItem[0].nSoundSize)
		PlaySoundFromFLASH(1);
	else
		PlaySoundFromFLASH(0);
	
}
void Task_Previous_Music(void)
{
	bMp3Play = 1;
	LD_Init_MP3();
	LD_Adjust_Volume(10);	
	if(nMp3Size==pAsrItem[1].nSoundSize)
		PlaySoundFromFLASH(0);
	else
		PlaySoundFromFLASH(1);
}
