#include "thread_init.h"
#include "robot_ctrl.h"
#include "led.h"

#if RT_THREAD
#include "rtthread.h"
#include "board.h"
#include "speaker.h"

static rt_thread_t main_core = RT_NULL,led_thread= RT_NULL,test_thread = RT_NULL;
static rt_timer_t led_work_timer = RT_NULL;
rt_sem_t led_wait_sem = RT_NULL;


static void main_process_entry(void* paramter);
static void led_work_callback(void* paramter);
static void led_thread_entry(void* paramter);

static void test_thread_entry(void* paramter);


void start_thread_entry(void* paramter)
{	
	led_work_timer = rt_timer_create("led_work_callback", /* 软件定时器的名称 */
                        led_work_callback,/* 软件定时器的回调函数 */
                        0,			/* 定时器超时函数的入口参数 */
                        200,   /* 软件定时器的超时时间(周期回调时间) */
                        RT_TIMER_FLAG_PERIODIC | RT_TIMER_FLAG_SOFT_TIMER);
                        /* 软件定时器模式 周期模式 */
  /* 启动定时器 */
  if (led_work_timer != RT_NULL) 
      rt_timer_start(led_work_timer);	
 	led_wait_sem =
	 rt_sem_create("led_wait_sem",
	                1,
	               RT_IPC_FLAG_FIFO);   
  
	if(led_wait_sem != RT_NULL)
		rt_kprintf("semaphore create success!\n");	 
	main_core = 
	  rt_thread_create("main_core",
	                   main_process_entry,
	                   RT_NULL,
	                   512,
	                   1,
	                   20);
	if(main_core != RT_NULL)
		rt_thread_startup(main_core);
	else
		return ;		

	if(Speaker_thread_Init() != -1){
		rt_kprintf("speaker thread init success\n");	
	}
	else
		return;	

	led_thread = 
	  rt_thread_create("led_process",
	                   led_thread_entry,
	                   RT_NULL,
	                   512,
	                   1,
	                   20);

	if(led_thread != RT_NULL)
		rt_thread_startup(led_thread);
	else
		return ;	
	test_thread = 
	  rt_thread_create("test_process",
	                   test_thread_entry,
	                   RT_NULL,
	                   128,
	                   2,
	                   20);

	if(test_thread != RT_NULL)
		rt_thread_startup(test_thread);
	else
		return ;	
}

static void main_process_entry(void* paramter)
{	
	while(1)
	{
		System_Process();
		rt_kprintf("main core\n");	
		rt_thread_yield();  //放弃剩余时间片
	}	
}
static void led_work_callback(void* paramter)
{
	rt_sem_release(led_wait_sem);//释放二值限号
	
//	rt_kprintf("led_work_callback runing\n");	
}
static void led_thread_entry(void* paramter)
{
	while(1)
	{
		rt_sem_take(led_wait_sem,	/* 获取信号量 */
                RT_WAITING_FOREVER); 	/* 等待时间：一直等 */	
		LedCtrl::GetInstance().Process(); 
		
//		rt_thread_delay(50);
		rt_thread_yield();  //放弃剩余时间片
		
		rt_kprintf("led_thread_entry runing\n");	
	}	
}
static void test_thread_entry(void* paramter)
{
	while(1)
	{	
//		rt_event_send(speak_task_event,ResposeSta);    	
		rt_thread_delay(500);
//		rt_thread_yield();  //放弃剩余时间片
//		
//		rt_kprintf("test_thread_entry runing\n");	
	}	
}

#endif
